#!/usr/bin/env bash
# shellcheck disable=SC2001

SCRIPTDIR=$(cd -- "$(dirname -- "${BASH_SOURCE[0]}")" &>/dev/null && pwd)
ROOTDIR="$(dirname "$SCRIPTDIR")"
if [ "$ROOTDIR" == "/" ]
then
  ROOTDIR=""
fi

## components versions
SERVICE_VERSION="1.52.0"
SURICATA_VERSION="7.0.42"
EBPF_TRACER_VERSION="1.20.2"
TLS_PACKET_CAPTURE_VERSION="1.0.18"

REFRESH_TOKEN=""
PROMPT_FOR_TOKEN="false"
PROMPT_FOR_CREDENTIALS="false"
PROXY_CREDENTIALS=""
DOWNLOAD_DISABLED="false"
REQUIRE_DOWNLOAD_DISABLED="false"
RELOCATE_INSTALLDIR="false"
RELOCATE_LOGDIR="false"
SET_MEMORY_ACCOUNTING="false"
MEMORY_ACCOUNTING_COMMAND="MemoryAccounting=yes"
SET_CPU_ACCOUNTING="false"
CPU_ACCOUNTING_COMMAND="CPUAccounting=yes"
CPU_CONFIGURATION_KEY="CPUQuota"

# mirror defaults
MIRROR_ALL_INTERFACES="false"
ENABLE_CUSTOM_FILTER="false"
ENABLE_TLS_CAPTURE="false"
RUN_NON_TLS_SERVER="false"
TLS_PACKET_CAPTURE_TLS_ENABLED="false"
SERVER_PORT="8001"
MIRROR_AFTER_TARGET="network-online.target"
PACKET_CAPTURE_AFTER_TARGET="network-online.target"
MIRROR_SERVICE_FILE="/usr/lib/systemd/system/suricata.service"
PACKET_CAPTURE_SERVICE_FILE="/usr/lib/systemd/system/tls-packet-capture.service"
ENABLE_REMOTE_INSPECTION_SURICATA="false"

# eBPF defaults
EXPORTER="OTLP"
REPORTING_PORT_DEFAULT="4317"
REMOTE_PORT_DEFAULT="5441"
REPORTING_PORT_TLS_DEFAULT="5443"
REMOTE_PORT_TLS_DEFAULT="5443"
REPLACE_EBPF_AFTER_TARGET="false"
ENABLE_JAVA_TLS_CAPTURE="false"
EBPF_SERVICE_FILE="/usr/lib/systemd/system/traceable-ebpf-tracer.service"
EBPF_SERVICE_DIR="${EBPF_SERVICE_FILE}.d"
EBPF_OVERRIDE_CONFIG_FILE=""

# TPA defaults
# 1 -> file path as input, 2-> base64 encoded values as input
TPA_TLS_MODE=""
REPLACE_TPA_AFTER_TARGET="false"
RELOCATE_DATADIR="false"
TPA_SERVICE_FILE="/usr/lib/systemd/system/traceable.service"
TPA_SERVICE_DIR="${TPA_SERVICE_FILE}.d"
TPA_TRACE_PROPAGATION_FORMAT=""
TPA_EXT_CAP_ENABLE_BLOCKING="false"
TPA_OVERRIDE_CONFIG_FILE=""
NO_PROXY="localhost,127.0.0.1"

# rpm versions
SERVICE_VERSION_RPM=$(echo "${SERVICE_VERSION}" | sed -e 's/-/\./g')
SURICATA_VERSION_RPM=$(echo "${SURICATA_VERSION}" | sed -e 's/-/\./g')
EBPF_TRACER_VERSION_RPM=$(echo "${EBPF_TRACER_VERSION}" | sed -e 's/-/\./g')
TLS_PACKET_CAPTURE_VERSION_RPM=$(echo "${TLS_PACKET_CAPTURE_VERSION}" | sed -e 's/-/\./g')

# install flags
INSTALL_TPA="false"
INSTALL_SURICATA="false"
INSTALL_EBPF="false"
SIGNING_VERIFICATION="true"
INSTALL_HTTP_PROXY="false"
UPDATE_AGENT_CONFIG_WITH_GRPC_TO_HTTP_PROXY="false"

checkErrorAndExitWithMessage() {
  # shellcheck disable=SC2181
  if [ $? -ne 0 ]
  then
    echo "Error occurred !!!"
    echo -e "\n$1"
    exit 1
  fi
}

print_help() {
  cat << EndOfHelpMessage

  NAME
    install - Utility to install and configure Traceable traffic mirroring components

  SYNOPSIS
    ./install.sh COMMAND help

  COMMANDS
    COMMAND is one of the following:

        mirror          Uses Packet capture for data collection
        ebpf            Uses eBPF for data collection
        tpa-only        Install TPA only
        ebpf-only       Install eBPF only
        http-proxy-only Install a gRPC to HTTP Proxy
        cleanup         Clean traceable deployments
EndOfHelpMessage
  exit
}

print_help_mirror() {
  cat << EndOfHelpMirrorMessage

  NAME
    install mirror - Use Packet capture for data collection

  SYNOPSIS
    ./install.sh mirror -e ENVIRONMENT -s SERVICE_NAME [-r REMOTE_ENDPOINT]
      [-i INTERFACE | --all-interfaces] [-f FILTER_OPTION] [-t]
      [--http-proxy HTTP_PROXY] [--https-proxy HTTPS_PROXY] [--no-proxy NO_PROXY]
      [--otlp-file-storage-dir OTLP_FILE_STORAGE_DIR]
      [--otlp-exporter-sending-queue-size OTLP_EXPORTER_SENDING_QUEUE_SIZE]
      [--tpa-loglevel TPA_LOGLEVEL] [--after-target AFTER_TARGET]
      [--tpa-log-max-size-mb TPA_LOGSIZE] [--tpa-log-max-backups TPA_LOGCOUNT]
      [--tpa-max-memory TPA_MAX_MEM] [ --tpa-override-config-file TPA_OVERRIDE_CONFIG_FILE]
      [--mirror-max-memory MIRROR_MAX_MEM]
      [--packet-capture-max-memory PACKET_CAPTURE_MAX_MEM]
      [--set-memory-accounting] [--remote-cert-path ROOT_CA_CERT]
      [--no-download [--install-dir INSTALLDIR] [--log-dir LOGDIR]
          [--data-dir DATADIR]]
      [--enable-tls-capture [--run-non-tls-server | --tls-key KEY
          --tls-cert CERT] [--server-port SERVER_PORT]]
      [--netscaler-remote-inspection]
      [--no-verify]
      [--with-grpc-to-http-proxy]

  EXAMPLES
    To capture traffic from 'eth0' interface and see it under 'myEnv'
    environment and 'mySvc' service on Traceable platform, run:

        $ ./install.sh mirror -e myEnv -s mySvc -i eth0

    To capture udp traffic from 'eth0' interface for port 7070, run:

        $ ./install.sh mirror -e myEnv -s mySvc -i eth0 -f "udp and port 7070"

    To capture traffic from all interfaces, run:

        $ ./install.sh mirror -e myEnv -s mySvc --all-interfaces

    To capture encrypted traffic from 'eth0' interface without tls server, run:

        $ ./install.sh mirror -e myEnv -s mySvc -i eth0 --enable-tls-capture
              --run-non-tls-server

    To capture encrypted traffic from 'eth0' interface using tls key path as
    '/home/key.crt', tls certificate path as '/home/cert.crt', run:

        $ ./install.sh mirror -e myEnv -s mySvc -i eth0 --enable-tls-capture
              --tls-key /home/key.crt --tls-cert /home/cert.crt

    To use the grpc to http proxy for egress calls to Traceable platform use:
        $ ./install.sh mirror -e myEnv -s mySvc -i eth0 --with-grpc-to-http-proxy <grpc-to-http-options>
    note: refer to ./install.sh http-proxy-only --help for <grpc-to-http-options>

  REQUIRED ARGUMENTS
    -e ENVIRONMENT
        Environment for Traceable Platform Agent.

    -s SERVICE_NAME
        Service name for traffic mirroring.

    Exactly one of these should be specified:

      -i INTERFACE
          Comma separated list of interface ID for mirrored traffic.

      --all-interfaces
          Mirror all interfaces. This flag is not compatible with the
          --enable-tls-capture flag.


  FLAGS
    -r REMOTE_ENDPOINT
        Remote endpoint for Traceable Platform Agent.
        Default is api.traceable.ai

    -f FILTER_OPTION
        Custom filter option for Mirroring.

    -t
        Prompt for token and overwrite token in file /etc/traceable/agent/token.

    --http-proxy HTTP_PROXY
        It will be used as the proxy URL for HTTP requests unless overridden by --no-proxy flag.

    --https-proxy HTTPS_PROXY
        It will be used as the proxy URL for HTTPS requests unless overridden by --no-proxy flag.    
    
    --no-proxy NO_PROXY
        It specifies a string that contains comma-separated values specifying hosts that should be excluded from proxying. 
	      Each value is represented by an IP address prefix (1.2.3.4), an IP address prefix in CIDR notation (1.2.3.4/8), 
        a domain name, or a special DNS label (*).

    --otlp-file-storage-dir OTLP_FILE_STORAGE_DIR
        OTLP file storage extension directory. This is to persist the retry
        queue for traces and metrics. Default value is /tmp

    --otlp-exporter-sending-queue-size OTLP_EXPORTER_SENDING_QUEUE_SIZE
        OTLP exporter sending queue size (in batches) for traces and metrics.
        This config is also used for the sending queue size. Default is 1000.

    --tpa-loglevel TPA_LOGLEVEL
        Logging level for the Traceable service. TPA_LOGLEVEL must be one of:
        info, warn, error, debug. Default value is info.

    --after-target AFTER_TARGET
        Targets after which services should be started during system
        reboot. Default value is network-online.target

    --tpa-log-max-size-mb TPA_LOGSIZE
        Maximum size in MB for log files created by the Traceable service.
        Default value is 10.

    --tpa-log-max-backups TPA_LOGCOUNT
        Maximum number of backup log files created by the Traceable service.
        Default value is 10.

    --tpa-max-memory TPA_MAX_MEM
        Maximum memory limit for traceable-agent service. Takes a memory size in
        bytes. If the value is suffixed with K, M, G or T, the specified memory
        size is parsed as Kilobytes, Megabytes, Gigabytes, or Terabytes
        (with the base 1024), respectively. Alternatively, a percentage value
        may be specified, which is taken relative to the installed physical
        memory on the system.

    --tpa-override-config-file TPA_OVERRIDE_CONFIG_FILE
        A yaml file containing override values for the traceable platform agent config. Config values
        defined directly using cmd line args such as --otlp-exporter-sending-queue-size which overwrite the
        main config will be overridden too. Config values in this file take precedence over the main config file
        and the cmd line args. Config values defined using environment variables will override it though.

    --mirror-max-memory MIRROR_MAX_MEM
        Maximum memory limit for mirror service. Takes a memory size in
        bytes. If the value is suffixed with K, M, G or T, the specified memory
        size is parsed as Kilobytes, Megabytes, Gigabytes, or Terabytes
        (with the base 1024), respectively. Alternatively, a percentage value
        may be specified, which is taken relative to the installed physical
        memory on the system.

    --packet-capture-max-memory PACKET_CAPTURE_MAX_MEM
        Maximum memory limit for packet capture service. Takes a memory size in
        bytes. If the value is suffixed with K, M, G or T, the specified memory
        size is parsed as Kilobytes, Megabytes, Gigabytes, or Terabytes
        (with the base 1024), respectively. Alternatively, a percentage value
        may be specified, which is taken relative to the installed physical
        memory on the system.

    --set-memory-accounting
        set MemoryAccounting to true for the services. This is required on VMs
        where DefaultMemoryAccounting property is not enabled.

    --remote-cert-path ROOT_CA_CERT
        Root CA used to sign certificate for Traceable Platform.

    --no-download
        Perform airgapped installation by using local packages to install
        Traceable and its components. When this flag is specified, the following
        optional flags can also be specified:

          --install-dir INSTALLDIR
              Relocate install directory (/etc) to INSTALLDIR during the rpm
              installation of the services.

          --log-dir LOGDIR
              Relocate log directories (/var/traceable/log, /var/log/suricata)
              to LOGDIR during rpm installation of the services.

          --data-dir DATADIR
              Relocate data directory (/var/traceable/persistence) to DATADIR
              during the rpm installation of Traceable service.

    --enable-tls-capture
        Enable tls packet capture for mirror mode. This flag is not compatible
        with --all-interfaces flag. When this flag is specified, the following
        optional flags can also be specified:

          Maximum one of these should be specified:

            --run-non-tls-server
                Run tcp server without tls. By default, tcp server runs on tls
                using the keys located in /var/traceable/certs.

            --tls-key KEY --tls-cert CERT
                Private key and cert for tls-packet capture package.

          --server-port SERVER_PORT
              Port where a tcp server will run when tls capture is enabled.
              Master keys will be sent to this tcp server in standard format.
              Default value is 8001.

    --netscaler-remote-inspection
        Use this flag when capturing mirror traffic using remote inspection
        feature of netscaler. When --netscaler-remote-inspection is enabled, use ip tunnel
        interfaces for capturing traffic.

    --no-verify
        Use this flag to skip signing signature verification

    --with-grpc-to-http-proxy
        Use this flag to additionally setup an Envoy proxy to convert grpc to http
        note: refer to ./install.sh http-proxy-only --help for <grpc-to-http-options>


EndOfHelpMirrorMessage
  exit
}

print_help_ebpf() {
  cat << EndOHelpEbpfMessage

  NAME
    install ebpf - Use eBPF for data collection

  SYNOPSIS
    ./install.sh ebpf -e ENVIRONMENT -s SERVICE_NAME [-r REMOTE_ENDPOINT]
      [--http-proxy HTTP_PROXY] [--https-proxy HTTPS_PROXY] [--no-proxy NO_PROXY]
      [-P PORTS] [-S SSL_DIRS] [-E EXCLUDE_PROCESSES] [--mode MODE]
      [-R MAX_RETURN_PROBES] [--remote-cert-path ROOT_CA_CERT] [-t]
      [--enable-java-tls-capture] [--enable-tracepoints] [--tpa-loglevel TPA_LOGLEVEL]
      [--tpa-after-target TPA_AFTER_TARGET] [--tpa-log-max-size-mb TPA_LOGSIZE]
      [--tpa-log-max-backups TPA_LOGCOUNT] [--tpa-max-memory TPA_MAX_MEM]
      [--tpa-max-cpu TPA_MAX_CPU]
      [--otlp-file-storage-dir OTLP_FILE_STORAGE_DIR]
      [--otlp-exporter-sending-queue-size OTLP_EXPORTER_SENDING_QUEUE_SIZE]
      [--otlp-max-connection-age OTLP_MAX_CONNECTION_AGE]
      [--tpa-override-config-file TPA_OVERRIDE_CONFIG_FILE]
      [--ebpf-loglevel EBPF_LOGLEVEL] [--ebpf-after-target EBPF_AFTER_TARGET]
      [--ebpf-log-max-size-mb EBPF_LOGSIZE]
      [--ebpf-log-max-backups EBPF_LOGCOUNT]
      [--ebpf-max-memory EBPF_MAX_MEM] [--set-memory-accounting]
      [--ebpf-max-cpu EBPF_MAX_CPU]
      [--ebpf-request-per-second-limit EBPF_REQUEST_LIMIT]
      [--ebpf-max-connection EBPF_MAX_CONNECTION]
      [--ebpf-go-memory-limit EBPF_GO_MEMORY_LIMIT]
      [--ebpf-libssl-prefixes EBPF_LIBSSL_PREFIXES]
      [--ebpf-override-config-file OVERRIDE_CONFIG_FILE]
      [--no-download [--install-dir INSTALLDIR] [--log-dir LOGDIR]
          [--data-dir DATADIR]]
      [--no-verify]
      [--with-grpc-to-http-proxy]


  EXAMPLES
    To capture traffic under 'myEnv' environment and 'mySvc' service on
    Traceable platform, run:

        $ ./install.sh ebpf -e myEnv -s mySvc

    To capture traffic from ports 5000, 5050, and 8080, run:

        $ ./install.sh ebpf -e myEnv -s mySvc -P 5000,5050,8080

    To install Traceable and its components without connecting to internet, run:

        $ ./install.sh ebpf -e myEnv -s mySvc --no-download

    To capture both ingress and egress traffic from port 5000, run:

        $ ./install.sh ebpf -e myEnv -s mySvc -P 5000 --mode ingress_and_egress

    To install Traceable and ebpf services with /opt instead of /etc directory
    using the airgapped package, run:

        $ ./install.sh ebpf -e myEnv -s mySvc --no-download --install-dir /opt

    To use the grpc to http proxy for egress calls to Traceable platform use:
        $ ./install.sh ebpf -e myEnv -s mySvc --with-grpc-to-http-proxy <grpc-to-http-options>
    note: refer to ./install.sh http-proxy-only --help for <grpc-to-http-options>


  REQUIRED ARGUMENTS
    -e ENVIRONMENT
        Environment for Traceable Platform Agent.

    -s SERVICE_NAME
        Service name for traffic mirroring.


  FLAGS
    -r REMOTE_ENDPOINT
        Remote endpoint for Traceable Platform Agent.
        Default is api.traceable.ai

    -P PORTS
        Comma-separated list of ports to be tracked.

    -S SSL_DIRS
        Comma-separated list of SSL directories.

    -E EXCLUDE_PROCESSES
        Comma-separated list of process names to be excluded from tracking.

    --mode MODE
        Traffic type to be captured. MODE must be one of: ingress, egress,
        ingress_and_egress. Default value is ingress.

    -R MAX_RETURN_PROBES
        Maximum number of return probes. Default value is 1.

    --remote-cert-path ROOT_CA_CERT
        Root CA used to sign certificate for Traceable Platform.

    -t
        Prompt for token and overwrite token in file /etc/traceable/agent/token.

    --http-proxy HTTP_PROXY
        It will be used as the proxy URL for HTTP requests unless overridden by --no-proxy flag.

    --https-proxy HTTPS_PROXY
        It will be used as the proxy URL for HTTPS requests unless overridden by --no-proxy flag.    
    
    --no-proxy NO_PROXY
        It specifies a string that contains comma-separated values specifying hosts that should be excluded from proxying. 
	      Each value is represented by an IP address prefix (1.2.3.4), an IP address prefix in CIDR notation (1.2.3.4/8), 
        a domain name, or a special DNS label (*).

    --enable-java-tls-capture
        Enable java tls capture.

    --enable-tracepoints
        Enable tracepoints.

    --tpa-loglevel TPA_LOGLEVEL
        Logging level for the Traceable service. TPA_LOGLEVEL must be one of:
        info, warn, error, debug. Default value is info.

    --tpa-after-target TPA_AFTER_TARGET
        Targets after which Traceable service should be started during system
        reboot. Default value is network-online.target

    --tpa-log-max-size-mb TPA_LOGSIZE
        Maximum size in MB for log files created by the Traceable service.
        Default value is 10.

    --tpa-log-max-backups TPA_LOGCOUNT
        Maximum number of backup log files created by the Traceable service.
        Default value is 10.

    --tpa-max-memory TPA_MAX_MEM
        Maximum memory limit for traceable-agent service. Takes a memory size in
        bytes. If the value is suffixed with K, M, G or T, the specified memory
        size is parsed as Kilobytes, Megabytes, Gigabytes, or Terabytes
        (with the base 1024), respectively. Alternatively, a percentage value
        may be specified, which is taken relative to the installed physical
        memory on the system.

      --tpa-max-cpu TPA_MAX_CPU
        Maximum CPU limit for the traceable service. Takes a CPU limit in
        percentage. Use values > 100% for allotting CPU time on more than one CPU.

    --otlp-file-storage-dir OTLP_FILE_STORAGE_DIR
        OTLP file storage extension directory. This is to persist the retry
        queue for traces and metrics. Default value is /tmp

    --otlp-exporter-sending-queue-size OTLP_EXPORTER_SENDING_QUEUE_SIZE
        OTLP exporter sending queue size (in batches) for traces and metrics.
        This config is also used for the sending queue size. Default is 1000.

    --tpa-override-config-file TPA_OVERRIDE_CONFIG_FILE
        A yaml file containing override values for the traceable platform agent config. Config values
        defined directly using cmd line args such as --otlp-exporter-sending-queue-size which overwrite the
        main config will be overridden too. Config values in this file take precedence over the main config file
        and the cmd line args. Config values defined using environment variables will override it though.

    --ebpf-loglevel EBPF_LOGLEVEL
        Logging level for the eBPF service. EBPF_LOGLEVEL must be one of: info,
        warn, error, debug, trace. Default value is info.

    --ebpf-after-target EBPF_AFTER_TARGET
        Targets after which eBPF service should be started during system
        reboot. Default value is network-online.target

    --ebpf-log-max-size-mb EBPF_LOGSIZE
        Maximum size in MB for log files created by the eBPF service.
        Default value is 10.

    --ebpf-log-max-backups EBPF_LOGCOUNT
        Maximum number of backup log files created by the eBPF service.
        Default value is 10.

    --ebpf-max-memory EBPF_MAX_MEM
        Maximum memory limit for the eBPF service. Takes a memory size in
        bytes. If the value is suffixed with K, M, G or T, the specified memory
        size is parsed as Kilobytes, Megabytes, Gigabytes, or Terabytes
        (with the base 1024), respectively. Alternatively, a percentage value
        may be specified, which is taken relative to the installed physical
        memory on the system.

    --set-memory-accounting
        Set MemoryAccounting to true for the services. This is required on VMs
        where DefaultMemoryAccounting property is not enabled.

    --ebpf-max-cpu EBPF_MAX_CPU
        Maximum CPU limit for the eBPF service. Takes a CPU limit in
        percentage. Use values > 100% for allotting CPU time on more than one CPU.

    --ebpf-request-per-second-limit EBPF_REQUEST_PER_SECOND_LIMIT
        Set the max number of requests that should be captured by the ebpf
        tracer in a second.

    --ebpf-max-connection EBPF_MAX_CONNECTION
        Set maximum number of connections tracked by the tracer at any point in time.

    --ebpf-go-memory-limit EBPF_GO_MEMORY_LIMIT
        Substitute of GOMEMLIMIT environment variable. Expects input in k8s units.
        The default value is 0, in which case no limit is set.
        Examples: 128974848, 129e6, 129M,  128974848000m, 123Mi

    --ebpf-libssl-prefixes EBPF_LIBSSL_PREFIXES
        Comma-separated list of SSL prefixes to attach eBPF probe.
        Examples: libssl.so, libssl3.so

    --ebpf-override-config-file EBPF_OVERRIDE_CONFIG_FILE
        A yaml file containing override values for the ebpf tracer config. Config values
        defined directly using cmd line args such as --libssl-prefixes will take precedence
        to those in the override config file.

    --otlp-max-connection-age OTLP_MAX_CONNECTION_AGE
        OTLP collector server in Traceable Platform Agent connection keepalive
        duration. Can be expressed as duration string,
        valid units are "ns", "us", "ms", "s", "m", and "h".
        Examples: 100ms = 100 milliseconds, 120s = 120 seconds,
        120m = 120 minutes, 2h = 2 hours. Default value is infinity.

    --no-download
        Perform airgapped installation by using local packages to install
        Traceable and its components. When this flag is specified, the following
        optional flags can also be specified:

          --install-dir INSTALLDIR
              Relocate install directory (/etc) to INSTALLDIR during the rpm
              installation of Traceable and eBPF services.

          --log-dir LOGDIR
              Relocate log directory (/var/traceable/log) to LOGDIR during
              the rpm installation of Traceable and eBPF services.

          --data-dir DATADIR
              Relocate data directory (/var/traceable/persistence) to DATADIR
              during the rpm installation of Traceable and eBPF services.

    --no-verify
        Use this flag to skip signing signature verification

    --with-grpc-to-http-proxy
        Use this flag to additionally setup an Envoy proxy to convert grpc to http
        note: refer to ./install.sh http-proxy-only --help for <grpc-to-http-options>

EndOHelpEbpfMessage
  exit
}

print_help_tpa_only() {
  cat << EndOfHelpTPAOnlyMessage

  NAME
    install tpa-only - install Traceable Platform Agent

  SYNOPSIS
    ./install.sh tpa-only -e ENVIRONMENT [-s SERVICE_NAME] [-r REMOTE_ENDPOINT]
      [--http-proxy HTTP_PROXY] [--https-proxy HTTPS_PROXY] [--no-proxy NO_PROXY]
      [--loglevel LOGLEVEL] [--remote-cert-path ROOT_CA_CERT] [-t | --raw-token TOKEN]
      [--after-target AFTER_TARGET] [--log-max-size-mb SIZE]
      [--log-max-backups COUNT] [--max-memory MAX_MEMORY] [--set-memory-accounting]
      [--max-cpu MAX_CPU]
      [--no-download [--install-dir INSTALLDIR] [--log-dir LOGDIR] [--data-dir DATADIR]]
      [--tls-endpoint ENDPOINT --tls-key KEY --tls-cert CERT --tls-root-cert ROOT_CA_CERT]
      [--otlp-file-storage-dir OTLP_FILE_STORAGE_DIR]
      [--otlp-exporter-sending-queue-size OTLP_EXPORTER_SENDING_QUEUE_SIZE]
      [--otlp-max-connection-age OTLP_MAX_CONNECTION_AGE]
      [--ext-cap-enable-blocking]
      [--tls-endpoint ENDPOINT --tls-key-base64 KEY --tls-cert-base64 CERT
          --tls-root-cert-base64 ROOT_CA_CERT]
      [--hsl-enabled] [--hsl-endpoint ENDPOINT --hsl-tls-cert CERT --hsl-tls-key KEY
          --hsl-max-queue-size NUMBER --hsl-buffer-size NUMBER --hsl-delimiter DELIMITER]
      [--no-verify] [--override-config-file TPA_OVERRIDE_CONFIG_FILE]


  EXAMPLES
    To configure TPA to capture traffic under 'myEnv' environment and 'mySvc'
    service on Traceable platform, run:

        $ ./install.sh tpa-only -e myEnv -s mySvc

    To install Traceable service with /opt instead of /etc directory using the
    airgapped package, run:

        $ ./install.sh tpa-only -e myEnv -s mySvc --no-download --install-dir /opt

    To install TPA with TLS configured at endpoint '10.120.0.3:5443', tls key
    path as '/home/key.crt', tls certificate path as '/home/cert.crt' and
    tls root certificate path as '/home/root_ca.crt', run:

        $ ./install.sh tpa-only -e myEnv --tls-endpoint "10.120.0.3:5443" --tls-key /home/key.crt --tls-cert /home/cert.crt --tls-root-cert /home/root_ca.crt

    To configure TPA to run an HSL listener running with tls you would provide the following options:
       $ ./install.sh tpa-only -e myEnv --hsl-enabled --hsl-tls-cert /home/cert.crt --hsl-tls-key /home/key.crt

    To use the grpc to http proxy for egress calls to Traceable platform use:
        $ ./install.sh tpa-only -e myEnv -s mySvc --with-grpc-to-http-proxy <grpc-to-http-options>
    note: refer to ./install.sh http-proxy-only --help for <grpc-to-http-options>

  REQUIRED ARGUMENTS
    -e ENVIRONMENT
        Environment for Traceable Platform Agent.


  FLAGS
    -s SERVICE_NAME
        Service name for traffic mirroring.

    -r REMOTE_ENDPOINT
        Remote endpoint for Traceable Platform Agent.
        Default is api.traceable.ai

    --loglevel LOGLEVEL
        Logging level for the Traceable service. LOGLEVEL must be one of: info,
        warn, error, debug. Default value is info.

    --remote-cert-path ROOT_CA_CERT
        Root CA used to sign certificate for Traceable Platform.

    -t
        Prompt for token and overwrite token in file /etc/traceable/agent/token.

    --http-proxy HTTP_PROXY
        It will be used as the proxy URL for HTTP requests unless overridden by --no-proxy flag.

    --https-proxy HTTPS_PROXY
        It will be used as the proxy URL for HTTPS requests unless overridden by --no-proxy flag.    
    
    --no-proxy NO_PROXY
        It specifies a string that contains comma-separated values specifying hosts that should be excluded from proxying. 
	      Each value is represented by an IP address prefix (1.2.3.4), an IP address prefix in CIDR notation (1.2.3.4/8), 
        a domain name, or a special DNS label (*).

    --raw-token RAW_TOKEN
        Provide the token directly to the script and overwrite token in file
        /etc/traceable/agent/token. This flag is not compatible with the -t flag

    --after-target AFTER_TARGET
        Targets after which Traceable service should be started during system
        reboot. Default value is network-online.target

    --log-max-size-mb SIZE
        Maximum size in MB for log files created by the Traceable service.
        Default value is 10.

    --log-max-backups COUNT
        Maximum number of backup log files created by the Traceable service.
        Default value is 10.

    --max-memory MAX_MEMORY
        Maximum memory limit for the Traceable service. Takes a memory size in
        bytes. If the value is suffixed with K, M, G or T, the specified memory
        size is parsed as Kilobytes, Megabytes, Gigabytes, or Terabytes
        (with the base 1024), respectively. Alternatively, a percentage value
        may be specified, which is taken relative to the installed physical
        memory on the system.

    --set-memory-accounting
        set MemoryAccounting to true for the services. This is required on VMs
        where DefaultMemoryAccounting property is not enabled.

    --max-cpu MAX_CPU
          Maximum CPU limit for the traceable service. Takes a CPU limit in
          percentage. Use values > 100% for allotting CPU time on more than one CPU.

    --otlp-file-storage-dir OTLP_FILE_STORAGE_DIR
        OTLP file storage extension directory. This is to persist the retry
        queue for traces and metrics. Default value is /tmp

    --otlp-exporter-sending-queue-size OTLP_EXPORTER_SENDING_QUEUE_SIZE
        OTLP exporter sending queue size (in batches) for traces and metrics.
        This config is also used for the retry queue size. Default is 1000.

    --otlp-max-connection-age OTLP_MAX_CONNECTION_AGE
        OTLP collector server in Traceable Platform Agent connection keepalive
        duration. Can be expressed as duration string,
        valid units are "ns", "us", "ms", "s", "m", and "h".
        Examples: 100ms = 100 milliseconds, 120s = 120 seconds,
        120m = 120 minutes, 2h = 2 hours. Default value is infinity.

    --ext-cap-enable-blocking
        Provide this flag for enabling blocking evaluation by
        Traceable Platform Agent's ext_cap.

    --trace-propagation-format TRACE_PROPOGATION_FORMAT
        Provide a trace propagation format to be used by Traceable
        Platform Agent. Default values are B3 and TRACECONTEXT

    --override-config-file TPA_OVERRIDE_CONFIG_FILE
        A yaml file containing override values for the traceable platform agent config. Config values
        defined directly using cmd line args such as --otlp-exporter-sending-queue-size which overwrite the
        main config will be overridden too. Config values in this file take precedence over the main config file
        and the cmd line args. Config values defined using environment variables will override it though.

    --no-download
        Perform airgapped installation by using local packages to install
        Traceable and its components. When this flag is specified, the following
        optional flags can also be specified:

          --install-dir INSTALLDIR
              Relocate install directory (/etc) to INSTALLDIR during the rpm
              installation of Traceable service.

          --log-dir LOGDIR
              Relocate log directory (/var/traceable/log) to LOGDIR during
              the rpm installation of Traceable service.

          --data-dir DATADIR
              Relocate data directory (/var/traceable/persistence) to DATADIR
              during the rpm installation of Traceable service.

    --no-verify
        Use this flag to skip signing signature verification


    Following flags can be specified together as a group or omitted altogether:

      --tls-endpoint ENDPOINT
          TLS Endpoint of Traceable Platform Agent.
          NOTE: Port must be provided as part of the endpoint.

      --tls-key KEY
          Private key for Traceable Platform Agent.

      --tls-cert CERT
          Certificate for Traceable Platform Agent.

      --tls-root-cert ROOT_CA_CERT
          Root CA used to sign certificate for Traceable Platform Agent.

    Following flags can be specified together as a group or omitted altogether:

      --tls-endpoint ENDPOINT
          TLS Endpoint of Traceable Platform Agent.
          NOTE: Port must be provided as part of the endpoint.

      --tls-key-base64 KEY
          Private key in the base64 encoded format for Traceable Platform Agent.

      --tls-cert-base64 CERT
          Certificate in the base64 encoded format for Traceable Platform Agent.

      --tls-root-cert-base64 ROOT_CA_CERT
          Root CA in the base64 encoded format used to sign certificate for
          Traceable Platform Agent.

EndOfHelpTPAOnlyMessage
  exit
}

print_help_ebpf_only() {
  cat << EndOfEbpfOnlyMessage

  NAME
    install ebpf-only - install eBPF

  SYNOPSIS
    ./install.sh ebpf-only -s SERVICE_NAME --tpa-endpoint ENDPOINT [-P PORTS]
      [-S SSL_DIRS] [-E EXCLUDE_PROCESSES] [-R MAX_RET_PROBES] [--mode MODE]
      [--tls-root-cert ROOT_CA_CERT] [--reporting-port REPORTING_PORT]
      [--remote-port REMOTE_PORT] [--exporter EXPORTER] [--loglevel LOGLEVEL]
      [--after-target AFTER_TARGET] [--log-max-size-mb SIZE]
      [--log-max-backups COUNT] [--enable-java-tls-capture] [--enable-tracepoints]
      [--max-memory MAX_MEMORY] [--set-memory-accounting]
      [--max-cpu MAX_CPU]
      [--request-per-second-limit REQUEST_LIMIT]
      [--max-connection MAX_CONNECTION]
      [--go-memory-limit GO_MEMORY_LIMIT]
      [--libssl-prefixes EBPF_LIBSSL_PREFIXES]
      [--override-config-file EBPF_OVERRIDE_CONFIG_FILE]
      [--no-download [--install-dir INSTALLDIR] [--log-dir LOGDIR]]
      [--no-verify]


  EXAMPLES
    To send mirrored traffic to '10.120.0.3' under 'mySvc' service name, run:

        $ ./install.sh ebpf-only -s mySvc --tpa-endpoint 10.120.0.3

    To mirror traffic only from ports 5000 and 5050, and send to '10.120.0.3'
    under 'mySvc' service name, run:

        $ ./install.sh ebpf-only -s mySvc --tpa-endpoint 10.120.0.3 -P 5000,5050

    To mirror all ingress and egress traffic and send to '10.120.0.3' under
    'mySvc' service name, run:

        $ ./install.sh ebpf-only -s mySvc --tpa-endpoint 10.120.0.3 --mode ingress_and_egress

    To install the ebpf service with /opt instead of /etc directory using
    the airgapped package, run:

        $ ./install.sh ebpf-only -s mySvc --tpa-endpoint 10.120.0.3 --no-download --install-dir /opt

    To create TLS connection to TPA running on '10.120.0.3' and send mirrored
    traffic under 'mySvc' service name, run:

        $ ./install.sh ebpf-only -s mySvc --tpa-endpoint 10.120.0.3 --tls-root-cert /home/ca_cert.crt

    In the above invocation, root CA file is the same as the one used to install
    Traceable Platform Agent on the address '10.120.0.3'

    To send mirrored traffic to '10.120.0.3:4317/path/of/zipkin' under 'mySvc'
    service name and configure Traceable to fetch configs from
    '10.120.0.3:5441', run:

        $ ./install.sh ebpf-only -s mySvc --tpa-endpoint 10.120.0.3 --exporter ZIPKIN --reporting-port 4317 --remote-port 5441


  REQUIRED ARGUMENTS
    -s SERVICE_NAME
        Service name for traffic mirroring.

    --tpa-endpoint ENDPOINT
        Endpoint of Traceable Platform Agent for eBPF.


  FLAGS
    -P PORTS
        Comma-separated list of ports to be tracked.

    -S SSL_DIRS
        Comma-separated list of SSL directories.

    -E EXCLUDE_PROCESSES
        Comma-separated list of process names to be excluded from tracking.

    -R MAX_RET_PROBES
        Maximum number of return probes. Default value is 1.

    --mode MODE
        Traffic type to be captured. MODE must be one of: ingress, egress,
        ingress_and_egress. Default value is ingress.

    --tls-root-cert ROOT_CA_CERT
        Root CA used to sign certificate for Traceable Platform Agent. eBPF
        uses this to validate Traceable Platform Agents certificate. If this
        value is provided, eBPF uses TLS encryption for data sent to TPA.

    --reporting-port REPORTING_PORT
        Reporting endpoint port of Traceable Platform Agent.  If --tls-root-cert
        is provided, default value is 5443. Otherwise, default value is 4317.

    --remote-port REMOTE_PORT
        Remote endpoint port of Traceable Platform Agent.  If --tls-root-cert
        is provided, default value is 5443. Otherwise, default value is 5441.

    --exporter EXPORTER
        eBPF span exporter. EXPORTER must be one of: ZIPKIN, OTLP.
        Default value is OTLP.

    --loglevel LOGLEVEL
        Logging level for the Traceable service. LOGLEVEL must be one of: info,
        warn, error, debug, trace. Default value is info.

    --after-target AFTER_TARGET
        Targets after which eBPF service should be started during system
        reboot. Default value is network-online.target

    --log-max-size-mb SIZE
        Maximum size in MB for log files created by the eBPF service.
        Default value is 10.

    --log-max-backups COUNT
        Maximum number of backup log files created by the eBPF service.
        Default value is 10.

    --enable-java-tls-capture
      Enable java tls capture.

    --enable-tracepoints
      Enable tracepoints.

    --max-memory MAX_MEMORY
        Maximum memory limit for the eBPF service. Takes a memory size in
        bytes. If the value is suffixed with K, M, G or T, the specified memory
        size is parsed as Kilobytes, Megabytes, Gigabytes, or Terabytes
        (with the base 1024), respectively. Alternatively, a percentage value
        may be specified, which is taken relative to the installed physical
        memory on the system.

    --set-memory-accounting
        Set MemoryAccounting to true for the services. This is required on VMs
        where DefaultMemoryAccounting property is not enabled.

    --max-cpu MAX_CPU
        Maximum CPU limit for the eBPF service. Takes a CPU limit in
        percentage. Use values > 100% for allotting CPU time on more than one CPU.

    --request-per-second-limit REQUEST_PER_SECOND_LIMIT
        Set the max number of requests that should be captured by the ebpf
        tracer in a second.

    --max-connection MAX_CONNECTION
        Set maximum number of connections tracked by the tracer at any point in time.

    --go-memory-limit GO_MEMORY_LIMIT
        Substitute of GOMEMLIMIT environment variable. Expects input in k8s units.
        The default value is 0, in which case no limit is set.
        Examples: 128974848, 129e6, 129M,  128974848000m, 123Mi

    --libssl-prefixes EBPF_LIBSSL_PREFIXES
        Comma-separated list of SSL prefixes to attach eBPF probe.
        Examples: libssl.so, libssl3.so

    --override-config-file EBPF_OVERRIDE_CONFIG_FILE
        A yaml file containing override values for the ebpf tracer config. Config values
        defined directly using cmd line args such as --libssl-prefixes will take precedence
        to those in the override config file.

    --no-download
        Perform airgapped installation by using local packages to install
        Traceable and its components. When this flag is specified, the following
        optional flags can also be specified:

          --install-dir INSTALLDIR
              Relocate install directory (/etc) to INSTALLDIR during the rpm
              installation of eBPF service.

          --log-dir LOGDIR
              Relocate log directory (/var/traceable/log) to LOGDIR during
              the rpm installation of eBPF service.

    --no-verify
        Use this flag to skip signing signature verification

EndOfEbpfOnlyMessage
  exit
}

print_help_http_proxy_only() {
  cat << EndOfHttpProxyOnlyOption

  NAME
    install http-proxy-only - install a gRPC to HTTP proxy for TPA communication

  SYNOPSIS
    ./install.sh http-proxy-only --platform-host HOST --platform-port PLATFORM_PORT
      [-p PORT]
      [--envoy-version MAJOR.MINOR.NANO]
      [--server-cert PATH_TO_SERVER_CERT] [--server-key PATH_TO_SERVER_KEY]
      [--proxy-host PROXY_HOST] [--proxy-port PROXY_PORT]
      [--proxy-credentials-encoded]


  EXAMPLES
    To install where traffic must go through an HTTP/1 only egress proxy before reaching the platform:
        $ ./install.sh http-proxy-only --platform-host api.traceable.ai --platform-port 443 --proxy-host myinternal.proxy --proxy-port 8080

    To configure the envoy proxy to run with certs use:
      $ ./install.sh http-proxy-only --platform-host api.traceable.ai --platform-port 443 --proxy-host myinternal.proxy --proxy-port 8080 \
          --server-cert /etc/envoy/my-server.crt --server-key /etc/envoy/my-server.key

          note: make sure to update tpa to be configured with this CA cert

    To control what version of Envoy is installed, compatibility with the latest Traceable envoy template
        $ ./install.sh http-proxy-only --platform-host api.traceable.ai --envoy-version 1.30.2

  REQUIRED ARGUMENTS
    --platform-host HOST
       Endpoint of Traceable platform

    --proxy-host PROXY_HOST
      Your Proxy ip or hostname

    --proxy-port PROXY_PORT
      The port your proxy expects to receive traffic on


  FLAGS
    -p port
        Port to run the proxy on, default: 80

    --envoy-version MAJOR.MINOR.NANO
       Envoy version release to install, check https://github.com/envoyproxy/envoy/tags for the latest version

    --server-crt PATH_TO_SERVER_CERT
        Cert file used by Envoy to use https instead

    --server-key PATH_TO_SERVER_KEY
        Key file used by Envoy to use https instead

    --proxy-host PROXY_HOST
        Host that egress traffic from Envoy will pass through before going to platform host

    --proxy-port PROXY_PORT
        Port of proxy host to connect to

    --proxy-credentials-encoded
        Base64 encoded string of proxy credentials, sent via Proxy Authorization: Basic <b64 value>

EndOfHttpProxyOnlyOption
exit
}

print_help_cleanup() {
  cat << EndOfCleanupMessage

  NAME
    cleanup - Remove Traceable deployments

  SYNOPSIS
    ./install.sh cleanup

EndOfCleanupMessage
  exit
}

check_input_empty_and_output_error() {
  if [ -z "$1" ]
  then
    echo "Value for $2 can not be empty."
    error_msg "$3"
  fi
}

check_file_exists() {
  if [ ! -s "$1" ]
  then
    echo "$2 file is empty or does not exist."
    error_msg "$3"
  fi
}

check_tpa_loglevel() {
  if [ -n "$1" ] && [ "$1" != "info" ] && [ "$1" != "warn" ] && [ "$1" != "error" ] && [ "$1" != "debug" ]
  then
    printf "loglevel must be one of: info, warn, error, debug. Default value is info. Received value is %s.\n" "$1"
    error_msg "$2"
  fi
}

check_ebpf_loglevel() {
  if [ -n "$1" ] && [ "$1" != "info" ] && [ "$1" != "warn" ] && [ "$1" != "error" ] && [ "$1" != "debug" ] && [ "$1" != "trace" ]
  then
    printf "loglevel must be one of: info, warn, error, debug, trace. Default value is info. Received value is %s.\n" "$1"
    error_msg "$2"
  fi
}

# ref: https://github.com/hypertrace/agent-config/blob/800c2631ff6026340ba8bee2681fa8f0cf5a9f75/proto/hypertrace/agent/config/v1/config.proto#L117
check_tpa_trace_propagation_format() {
  if [ -n "$1" ] && [ "$1" != "B3" ] && [ "$1" != "TRACECONTEXT" ]
  then
    printf "trace-propagation-format must be one of: B3, TRACECONTEXT. By default both B3 and TRACECONTEXT are supported. Received value is %s.\n" "$1"
    error_msg "$2"
  fi
}

check_no_download() {
  if [ "$RELOCATE_INSTALLDIR" == "true" ] || [ "$RELOCATE_LOGDIR" == "true" ] || [ "$RELOCATE_DATADIR" == "true" ]
  then
    if [ "$DOWNLOAD_DISABLED" != "true" ]
    then
      printf "Please provide --no-download flag before you proceed, and make sure you are using the airgapped tar."
      error_msg "$1"
    fi
    if [ "$PACKAGE_MANAGER" != "yum" ] && [ "$PACKAGE_MANAGER" != "rpm" ]
    then
      printf "Relocation flags can be used only with rpm package manager."
      error_msg "$1"
    fi
  fi

  if [ "$REQUIRE_DOWNLOAD_DISABLED" == "true" ] && [ "$DOWNLOAD_DISABLED" != "true" ]
  then
    printf "Please provide --no-download flag before you proceed, and make sure you are using the airgapped tar."
    error_msg "$1"
  fi
}

check_mirror_mode() {
  if [ -n "$1" ] && [ "$1" != "ingress" ] && [ "$1" != "egress" ] && [ "$1" != "ingress_and_egress" ]
  then
    printf "mode must be one of: ingress, egress, ingress_and_egress. Default value is ingress. Received value is %s.\n" "$1"
    error_msg "$2"
  fi
}

create_otlp_file_storage_dir() {
  if [ -n "$1" ]
  then
    mkdir -p "$1" || (echo "unable to create $1" && exit)
  fi
}

set_package_manager() {
  if command -v apt >/dev/null;
  then
    export DEBIAN_FRONTEND=noninteractive
    PACKAGE_MANAGER="apt"
  elif command -v yum >/dev/null;
  then
    PACKAGE_MANAGER="yum"
  elif command -v rpm >/dev/null;
  then
    # In SUSE linux, rpm package manager is available but yum is not. In such
    # cases, we will use rpm for installation using the airgapped tar.
    PACKAGE_MANAGER="rpm"
    REQUIRE_DOWNLOAD_DISABLED="true"
    echo "Note: only offline installation is supported for this Linux distribution."
  else
    echo "Distribution is not supported."
    exit 1
  fi
}

set_package_url() {
  if [ -n "$1" ]
  then
    PACKAGE_URL="$1"
    SIGNING_VERIFICATION="false"
    return
  elif [ "$PACKAGE_MANAGER" == "apt" ]
  then
    PACKAGE_URL="https://packages.traceable.ai/ubuntu/"
    return
  elif [ -s "/etc/redhat-release" ]
  then
    CENTOS_VERSION=$(grep -o "[0-9]" /etc/redhat-release | head -1)
    if [ "$CENTOS_VERSION" == "8" ]
    then
      PACKAGE_URL="https://packages.traceable.ai/centos8/"
      return
    fi
  fi
  PACKAGE_URL="https://packages.traceable.ai/centos7/"
}

get_interface_options() {
  TLS_PACKET_CAPTURE_INTERFACE_OPTIONS=""
  INTERFACE_COUNT=0
  CAPTURE_INTERFACE_CONFIG="-i "
  if [ "$ENABLE_REMOTE_INSPECTION_SURICATA" == "true" ]
  then
    CAPTURE_INTERFACE_CONFIG="--pcap="
  fi
  if [ "$MIRROR_ALL_INTERFACES" == "true" ]
  then
    # shellcheck disable=SC2045
    for interface in $(ls /sys/class/net)
    do
      if [ "$interface" != "lo" ]
      then
        TLS_PACKET_CAPTURE_INTERFACE_OPTIONS="$CAPTURE_INTERFACE_CONFIG$interface $TLS_PACKET_CAPTURE_INTERFACE_OPTIONS"
        INTERFACE_COUNT=$((INTERFACE_COUNT + 1))
      fi
    done
  else
    for interface in $(echo "$INTERFACE" | tr "," "\n")
    do
      TLS_PACKET_CAPTURE_INTERFACE_OPTIONS="$CAPTURE_INTERFACE_CONFIG$interface $TLS_PACKET_CAPTURE_INTERFACE_OPTIONS"
      INTERFACE_COUNT=$((INTERFACE_COUNT + 1))
    done
  fi
}

error_msg() {
  printf "For more details, run:\n\n\t$ ./install.sh %s help\n\n" "$1"
  exit 1
}

parse_input() {
  if [ "$#" -eq "0" ]
  then
    echo "No input provided."
    error_msg
  fi

  local cmd_type="$1"
  case $cmd_type in
    mirror)
      parse_input_mirror "${@:2}"
      ;;
    ebpf)
      parse_input_ebpf "${@:2}"
      ;;
    tpa-only)
      parse_input_tpa_only "${@:2}"
      ;;
    ebpf-only)
      parse_input_ebpf_only "${@:2}"
      ;;
    http-proxy-only)
      parse_input_http_proxy_only "${@:2}"
      ;;
    cleanup)
      parse_input_cleanup "${@:2}"
      ;;
    help)
      print_help
      ;;
    *) #unknown option
      echo "Unknown cmd type: $cmd_type."
      error_msg
      ;;
  esac
}

parse_input_mirror() {
  INSTALL_TPA="true"
  INSTALL_SURICATA="true"

  local package_url
  while [[ $# -gt 0 ]]
  do
    key="$1"
    case $key in
      -e|--environment)
        ENVIRONMENT="$2"
        shift
        shift
        ;;
      -s|--service-name)
        SERVICE_NAME="$2"
        shift
        shift
        ;;
      -r|--remote-endpoint)
        REMOTE_ENDPOINT="$2"
        shift
        shift
        ;;
      -i|--interface)
        INTERFACE="$2"
        shift
        shift
        ;;
      --all-interfaces)
        MIRROR_ALL_INTERFACES="true"
        shift
        ;;
      -f|--filter-option)
        FILTER_OPTION="$2"
        ENABLE_CUSTOM_FILTER="true"
        shift
        shift
        ;;
      -t|--token)
        PROMPT_FOR_TOKEN="true"
        shift
        ;;
      --otlp-file-storage-dir)
        OTLP_FILE_STORAGE_DIR="$2"
        shift
        shift
        ;;
      --otlp-exporter-sending-queue-size)
        OTLP_EXPORTER_SENDING_QUEUE_SIZE="$2"
        shift
        shift
        ;;
      --tpa-loglevel)
        TPA_LOGLEVEL="$2"
        shift
        shift
        ;;
      --after-target)
        REPLACE_TPA_AFTER_TARGET="true"
        TPA_AFTER_TARGET="$2"
        MIRROR_AFTER_TARGET="$2"
        PACKET_CAPTURE_AFTER_TARGET="$2"
        shift
        shift
        ;;
      --tpa-log-max-size-mb)
        TPA_LOG_MAX_SIZE_MB="$2"
        shift
        shift
        ;;
      --tpa-log-max-backups)
        TPA_LOG_MAX_BACKUPS="$2"
        shift
        shift
        ;;
      --tpa-max-memory)
        TPA_MAX_MEM="$2"
        shift
        shift
        ;;
      --tpa-override-config-file)
        TPA_OVERRIDE_CONFIG_FILE="$2"
        shift
        shift
        ;;
      --mirror-max-memory)
        MIRROR_MAX_MEM="$2"
        shift
        shift
        ;;
      --packet-capture-max-memory)
        PACKET_CAPTURE_MAX_MEM="$2"
        shift
        shift
        ;;
      --set-memory-accounting)
        SET_MEMORY_ACCOUNTING="true"
        shift
        ;;
      --remote-cert-path)
        REMOTE_CERT_PATH="$2"
        shift
        shift
        ;;
      --no-download)
        DOWNLOAD_DISABLED="true"
        shift
        ;;
      --install-dir)
        NEW_INSTALLDIR="$2"
        RELOCATE_INSTALLDIR="true"
        shift
        shift
        ;;
      --log-dir)
        NEW_LOGDIR="$2"
        RELOCATE_LOGDIR="true"
        shift
        shift
        ;;
      --data-dir)
        NEW_DATADIR="$2"
        RELOCATE_DATADIR="true"
        shift
        shift
        ;;
      --enable-tls-capture)
        ENABLE_TLS_CAPTURE="true"
        shift
        ;;
      --run-non-tls-server)
        RUN_NON_TLS_SERVER="true"
        shift
        ;;
      --tls-key)
        TLS_KEY_FILE="$2"
        TLS_PACKET_CAPTURE_TLS_ENABLED="true"
        shift
        shift
        ;;
      --tls-cert)
        TLS_CERT_FILE="$2"
        TLS_PACKET_CAPTURE_TLS_ENABLED="true"
        shift
        shift
        ;;
      --server-port)
        SERVER_PORT="$2"
        shift
        shift
        ;;
      -p|--package-url)
        package_url="$2"
        shift
        shift
        ;;
      --netscaler-remote-inspection)
        ENABLE_REMOTE_INSPECTION_SURICATA="true"
        shift
        ;;
      --no-verify)
        SIGNING_VERIFICATION="false"
        shift
        ;;
      --http-proxy)
        HTTP_PROXY="$2"
        shift
        shift
        ;;
      --https-proxy)
        HTTPS_PROXY="$2"
        shift
        shift
        ;;
      --no-proxy)
        NO_PROXY="$2"
        shift
        shift
        ;;
      --with-grpc-to-http-proxy)
        UPDATE_AGENT_CONFIG_WITH_GRPC_TO_HTTP_PROXY="true"
        shift
        parse_input_http_proxy_only "$@"
        break
        ;;
      help)
        print_help_mirror
        ;;
      *) # unknown option
        printf "Unknown option %s.\n" "$key"
        error_msg "mirror"
        ;;
    esac
  done
  cleanup_traceable_deployments
  check_input_empty_and_output_error "$ENVIRONMENT" "environment" "mirror"
  check_input_empty_and_output_error "$SERVICE_NAME" "service_name" "mirror"
  check_tpa_loglevel "$TPA_LOGLEVEL" "mirror"
  check_no_download "mirror"
  set_package_url "$package_url"
  create_otlp_file_storage_dir "$OTLP_FILE_STORAGE_DIR"
  get_interface_options

  if [ "$MIRROR_ALL_INTERFACES" == "true" ]
  then
    if [ -n "$INTERFACE" ]
    then
      echo "Exactly one of -i and --all-interfaces should be specified."
      error_msg "mirror"
    fi
  else
    check_input_empty_and_output_error "$INTERFACE" "interface" "mirror"
  fi

  if [ "$ENABLE_TLS_CAPTURE" == "true" ]
  then
    if [ "$ENABLE_REMOTE_INSPECTION_SURICATA" == "true" ]
    then
      echo "--netscaler-remote-inspection can not be used with tls capture mode"
      error_msg "mirror"
    fi
    if [ "$INTERFACE_COUNT" != "1" ]
    then
      printf "The number of interfaces provided with --enable-tls-capture flag should be exactly one. Received: %s. Interface options: %s.\n" "$INTERFACE_COUNT" "$TLS_PACKET_CAPTURE_INTERFACE_OPTIONS"
      error_msg "mirror"
    fi

    if [ "$RUN_NON_TLS_SERVER" == "true" ]
    then
      if [ "$TLS_PACKET_CAPTURE_TLS_ENABLED" == "true" ]
      then
        echo "You can either provide [--run-non-tls-server] or [--tls-key KEY --tls-cert CERT], but not both."
        error_msg "mirror"
      fi
      TLS_PACKET_CAPTURE_TLS_OPTIONS="-N"
    elif [ "$TLS_PACKET_CAPTURE_TLS_ENABLED" == "true" ]
    then
      check_file_exists "$TLS_KEY_FILE" "tls key" "mirror"
      check_file_exists "$TLS_CERT_FILE" "tls cert" "mirror"
      TLS_PACKET_CAPTURE_TLS_OPTIONS="-K $TLS_KEY_FILE -C $TLS_CERT_FILE"
    else
      mkdir -p /var/traceable/certs
      echo -e "\nUS\n.\n.\n.\nmirror\nselfcert.traceable.ai\n" | openssl req -x509 -newkey rsa:4096 -keyout /var/traceable/certs/key.pem -out /var/traceable/certs/cert.pem -sha256 -days 365 -nodes >/dev/null 2>&1
      check_file_exists "/var/traceable/certs/key.pem" "tls key" "mirror"
      check_file_exists "/var/traceable/certs/cert.pem" "tls cert" "mirror"
      TLS_PACKET_CAPTURE_TLS_OPTIONS="-K /var/traceable/certs/key.pem -C /var/traceable/certs/cert.pem"
    fi
  fi
}

parse_input_ebpf() {
  INSTALL_TPA="true"
  INSTALL_EBPF="true"

  local package_url
  while [[ $# -gt 0 ]]
  do
    key="$1"
    case $key in
      -e|--environment)
        ENVIRONMENT="$2"
        shift
        shift
        ;;
      -s|--service-name)
        SERVICE_NAME="$2"
        shift
        shift
        ;;
      -r|--remote-endpoint)
        REMOTE_ENDPOINT="$2"
        shift
        shift
        ;;
      -P|--ports)
        PORTS="$2"
        shift
        shift
        ;;
      -S|--ssl-dirs)
        SSL_DIRS="$2"
        shift
        shift
        ;;
      -E|--exclude-processes)
        EXCLUDE_PROCESSES="$2"
        shift
        shift
        ;;
      --mode)
        MODE="$2"
        shift
        shift
        ;;
      -R|--max-return-probes)
        MAX_RETURN_PROBES="$2"
        shift
        shift
        ;;
      --remote-cert-path)
        REMOTE_CERT_PATH="$2"
        shift
        shift
        ;;
      -t|--token)
        PROMPT_FOR_TOKEN="true"
        shift
        ;;
      --enable-java-tls-capture)
        ENABLE_JAVA_TLS_CAPTURE="true"
        shift
        ;;
      --enable-tracepoints)
        ENABLE_TRACEPOINTS="true"
        shift
        ;;
      --tpa-loglevel)
        TPA_LOGLEVEL="$2"
        shift
        shift
        ;;
      --tpa-after-target)
        REPLACE_TPA_AFTER_TARGET="true"
        TPA_AFTER_TARGET="$2"
        shift
        shift
        ;;
      --tpa-log-max-size-mb)
        TPA_LOG_MAX_SIZE_MB="$2"
        shift
        shift
        ;;
      --tpa-log-max-backups)
        TPA_LOG_MAX_BACKUPS="$2"
        shift
        shift
        ;;
      --tpa-max-memory)
        TPA_MAX_MEM="$2"
        shift
        shift
        ;;
      --tpa-max-cpu)
        TPA_MAX_CPU="$2"
        shift
        shift
        ;;
      --otlp-file-storage-dir)
        OTLP_FILE_STORAGE_DIR="$2"
        shift
        shift
        ;;
      --otlp-exporter-sending-queue-size)
        OTLP_EXPORTER_SENDING_QUEUE_SIZE="$2"
        shift
        shift
        ;;
      --tpa-override-config-file)
        TPA_OVERRIDE_CONFIG_FILE="$2"
        shift
        shift
        ;;
      --ebpf-loglevel)
        EBPF_LOGLEVEL="$2"
        shift
        shift
        ;;
      --ebpf-after-target)
        REPLACE_EBPF_AFTER_TARGET="true"
        EBPF_AFTER_TARGET="$2"
        shift
        shift
        ;;
      --ebpf-log-max-size-mb)
        EBPF_LOG_MAX_SIZE_MB="$2"
        shift
        shift
        ;;
      --ebpf-log-max-backups)
        EBPF_LOG_MAX_BACKUPS="$2"
        shift
        shift
        ;;
      --ebpf-max-memory)
        EBPF_MAX_MEM="$2"
        shift
        shift
        ;;
      --set-memory-accounting)
        SET_MEMORY_ACCOUNTING="true"
        shift
        ;;
      --ebpf-max-cpu)
        EBPF_MAX_CPU="$2"
        shift
        shift
        ;;
      --no-download)
        DOWNLOAD_DISABLED="true"
        shift
        ;;
      --install-dir)
        NEW_INSTALLDIR="$2"
        RELOCATE_INSTALLDIR="true"
        shift
        shift
        ;;
      --log-dir)
        NEW_LOGDIR="$2"
        RELOCATE_LOGDIR="true"
        shift
        shift
        ;;
      --data-dir)
        NEW_DATADIR="$2"
        RELOCATE_DATADIR="true"
        shift
        shift
        ;;
      -p|--package-url)
        package_url="$2"
        shift
        shift
        ;;
      --otlp-max-connection-age)
        OTLP_MAX_CONNECTION_AGE="$2"
        shift
        shift
        ;;
      --ebpf-request-per-second-limit)
        EBPF_REQUEST_PER_SECOND_LIMIT="$2"
        shift
        shift
        ;;
      --ebpf-max-connection)
        EBPF_MAX_CONNECTION="$2"
        shift
        shift
        ;;
      --ebpf-go-memory-limit)
        EBPF_GO_MEMORY_LIMIT="$2"
        shift
        shift
        ;;
      --ebpf-libssl-prefixes)
        EBPF_LIBSSL_PREFIXES="$2"
        shift
        shift
        ;;
      --ebpf-override-config-file)
        EBPF_OVERRIDE_CONFIG_FILE="$2"
        shift
        shift
        ;;
      --no-verify)
        SIGNING_VERIFICATION="false"
        shift
        ;;
      --http-proxy)
        HTTP_PROXY="$2"
        shift
        shift
        ;;
      --https-proxy)
        HTTPS_PROXY="$2"
        shift
        shift
        ;;
      --no-proxy)
        NO_PROXY="$2"
        shift
        shift
        ;;
      --with-grpc-to-http-proxy)
        UPDATE_AGENT_CONFIG_WITH_GRPC_TO_HTTP_PROXY="true"
        shift
        parse_input_http_proxy_only "$@"
        break
        ;;
      help)
        print_help_ebpf
        ;;
      *) # unknown option
        printf "Unknown option %s.\n" "$key"
        error_msg "ebpf"
        ;;
    esac
  done
  cleanup_traceable_deployments
  check_input_empty_and_output_error "$ENVIRONMENT" "environment" "ebpf"
  check_input_empty_and_output_error "$SERVICE_NAME" "service_name" "ebpf"
  check_ebpf_loglevel "$EBPF_LOGLEVEL" "ebpf"
  check_tpa_loglevel "$TPA_LOGLEVEL" "ebpf"
  check_mirror_mode "$MODE" "ebpf"
  check_no_download "ebpf"
  set_package_url "$package_url"
  create_otlp_file_storage_dir "$OTLP_FILE_STORAGE_DIR"
}

parse_input_tpa_only() {
  INSTALL_TPA="true"

  local package_url
  while [[ $# -gt 0 ]]
  do
    key="$1"
    case $key in
      -e|--environment)
        ENVIRONMENT="$2"
        shift
        shift
        ;;
      -s|--service-name)
        SERVICE_NAME="$2"
        shift
        shift
        ;;
      -r|--remote-endpoint)
        REMOTE_ENDPOINT="$2"
        shift
        shift
        ;;
      --loglevel)
        TPA_LOGLEVEL="$2"
        shift
        shift
        ;;
      --remote-cert-path)
        REMOTE_CERT_PATH="$2"
        shift
        shift
        ;;
      -t|--token)
        PROMPT_FOR_TOKEN="true"
        shift
        ;;
      --raw-token)
        REFRESH_TOKEN="$2"
        shift
        shift
        ;;
      --after-target)
        REPLACE_TPA_AFTER_TARGET="true"
        TPA_AFTER_TARGET="$2"
        shift
        shift
        ;;
      --log-max-size-mb)
        TPA_LOG_MAX_SIZE_MB="$2"
        shift
        shift
        ;;
      --log-max-backups)
        TPA_LOG_MAX_BACKUPS="$2"
        shift
        shift
        ;;
      --max-memory)
        TPA_MAX_MEM="$2"
        shift
        shift
        ;;
      --max-cpu)
        TPA_MAX_CPU="$2"
        shift
        shift
        ;;
      --set-memory-accounting)
        SET_MEMORY_ACCOUNTING="true"
        shift
        ;;
      --no-download)
        DOWNLOAD_DISABLED="true"
        shift
        ;;
      --install-dir)
        NEW_INSTALLDIR="$2"
        RELOCATE_INSTALLDIR="true"
        shift
        shift
        ;;
      --log-dir)
        NEW_LOGDIR="$2"
        RELOCATE_LOGDIR="true"
        shift
        shift
        ;;
      --data-dir)
        NEW_DATADIR="$2"
        RELOCATE_DATADIR="true"
        shift
        shift
        ;;
      --tls-endpoint)
        TLS_ENDPOINT="$2"
        shift
        shift
        ;;
      --tls-key)
        TLS_KEY_FILE="$2"
        TPA_TLS_MODE="1"
        shift
        shift
        ;;
      --tls-cert)
        TLS_CERT_FILE="$2"
        TPA_TLS_MODE="1"
        shift
        shift
        ;;
      --tls-root-cert)
        TLS_ROOT_CERT_FILE="$2"
        TPA_TLS_MODE="1"
        shift
        shift
        ;;
      --tls-key-base64)
        TLS_KEY_BASE64="$2"
        TPA_TLS_MODE="2"
        shift
        shift
        ;;
      --tls-cert-base64)
        TLS_CERT_BASE64="$2"
        TPA_TLS_MODE="2"
        shift
        shift
        ;;
      --tls-root-cert-base64)
        TLS_ROOT_CERT_BASE64="$2"
        TPA_TLS_MODE="2"
        shift
        shift
        ;;
      --otlp-file-storage-dir)
        OTLP_FILE_STORAGE_DIR="$2"
        shift
        shift
        ;;
      --otlp-exporter-sending-queue-size)
        OTLP_EXPORTER_SENDING_QUEUE_SIZE="$2"
        shift
        shift
        ;;
      -p|--package-url)
        package_url="$2"
        shift
        shift
        ;;
      --ext-cap-enable-blocking)
        TPA_EXT_CAP_ENABLE_BLOCKING="true"
        shift
        ;;
      --trace-propagation-format)
        TPA_TRACE_PROPAGATION_FORMAT="$2"
        shift
        shift
        ;;
      --otlp-max-connection-age)
        OTLP_MAX_CONNECTION_AGE="$2"
        shift
        shift
        ;;
      --hsl-enabled)
        HSL_ENABLED="true"
        shift
        ;;
      --hsl-endpoint)
        HSL_ENDPOINT="$2"
        shift
        shift
        ;;
      --hsl-tls-cert)
        HSL_TLS_CERT="$2"
        shift
        shift
        ;;
      --hsl-tls-key)
        HSL_TLS_KEY="$2"
        shift
        shift
        ;;
      --hsl-max-queue-size)
        HSL_MAX_QUEUE_SIZE="$2"
        shift
        shift
        ;;
      --hsl-buffer-size)
        HSL_BUFFER_SIZE="$2"
        shift
        shift
        ;;
      --hsl-delimiter)
        HSL_DELIMITER="$2"
        shift
        shift
        ;;
      --no-verify)
        SIGNING_VERIFICATION="false"
        shift
        ;;
      --override-config-file)
        TPA_OVERRIDE_CONFIG_FILE="$2"
        shift
        shift
        ;;
      --http-proxy)
        HTTP_PROXY="$2"
        shift
        shift
        ;;
      --https-proxy)
        HTTPS_PROXY="$2"
        shift
        shift
        ;;
      --no-proxy)
        NO_PROXY="$2"
        shift
        shift
        ;;
      --with-grpc-to-http-proxy)
        UPDATE_AGENT_CONFIG_WITH_GRPC_TO_HTTP_PROXY="true"
        shift
        parse_input_http_proxy_only "$@"
        break
        ;;
      help)
        print_help_tpa_only
        ;;
      *) # unknown option
        printf "Unknown option %s.\n" "$key"
        error_msg "tpa-only"
        ;;
    esac
  done
  cleanup_traceable_deployments
  check_input_empty_and_output_error "$ENVIRONMENT" "environment" "tpa-only"
  check_tpa_loglevel "$TPA_LOGLEVEL" "tpa-only"
  check_tpa_trace_propagation_format "$TPA_TRACE_PROPAGATION_FORMAT" "tpa-only"
  check_no_download "tpa-only"
  set_package_url "$package_url"
  create_otlp_file_storage_dir "$OTLP_FILE_STORAGE_DIR"

  if [ -n "$TPA_TLS_MODE" ]
  then
    check_input_empty_and_output_error "$TLS_ENDPOINT" "tls endpoint" "tpa-only"
    if [ "$TPA_TLS_MODE" == "1" ]
    then
      check_input_empty_and_output_error "$TLS_KEY_FILE" "tls key" "tpa-only"
      check_input_empty_and_output_error "$TLS_CERT_FILE" "tls certificate" "tpa-only"
      check_input_empty_and_output_error "$TLS_ROOT_CERT_FILE" "tls root certificate" "tpa-only"
    else
      # try to decode the base64 certificates if they are provided
      decode_tpa_certs
      check_input_empty_and_output_error "$TLS_KEY_BASE64" "tls key base64" "tpa-only"
      check_input_empty_and_output_error "$TLS_CERT_BASE64" "tls certificate base64" "tpa-only"
      check_input_empty_and_output_error "$TLS_ROOT_CERT_BASE64" "tls root certificate base64" "tpa-only"
    fi
  elif [ -n "$TLS_ENDPOINT" ]
  then
    echo "--tls-endpoint needs more flags to be passed."
    error_msg "tpa-only"
  fi

  if [ "$PROMPT_FOR_TOKEN" == "true" ] && [ -n "$REFRESH_TOKEN" ]
  then
      echo "At most one of -t and --raw-token should be specified."
      error_msg "tpa-only"
  fi
}

parse_input_ebpf_only() {
  INSTALL_EBPF="true"

  local package_url
  while [[ $# -gt 0 ]]
  do
    key="$1"
    case $key in
      -s|--service-name)
        SERVICE_NAME="$2"
        shift
        shift
        ;;
      --tpa-endpoint)
        TPA_ENDPOINT="$2"
        shift
        shift
        ;;
      -P|--ports)
        PORTS="$2"
        shift
        shift
        ;;
      -S|--ssl-dirs)
        SSL_DIRS="$2"
        shift
        shift
        ;;
      -E|--exclude-processes)
        EXCLUDE_PROCESSES="$2"
        shift
        shift
        ;;
      -R|--max-return-probes)
        MAX_RETURN_PROBES="$2"
        shift
        shift
        ;;
      --mode)
        MODE="$2"
        shift
        shift
        ;;
      --tls-root-cert)
        CA_CERT_PATH="$2"
        shift
        shift
        ;;
      --reporting-port)
        REPORTING_PORT="$2"
        shift
        shift
        ;;
      --remote-port)
        REMOTE_PORT="$2"
        shift
        shift
        ;;
      --exporter)
        EXPORTER="$2"
        shift
        shift
        ;;
      --loglevel)
        EBPF_LOGLEVEL="$2"
        shift
        shift
        ;;
      --after-target)
        REPLACE_EBPF_AFTER_TARGET="true"
        EBPF_AFTER_TARGET="$2"
        shift
        shift
        ;;
      --log-max-size-mb)
        EBPF_LOG_MAX_SIZE_MB="$2"
        shift
        shift
        ;;
      --log-max-backups)
        EBPF_LOG_MAX_BACKUPS="$2"
        shift
        shift
        ;;
      --max-memory)
        EBPF_MAX_MEM="$2"
        shift
        shift
        ;;
      --set-memory-accounting)
        SET_MEMORY_ACCOUNTING="true"
        shift
        ;;
      --max-cpu)
        EBPF_MAX_CPU="$2"
        shift
        shift
        ;;
      --enable-java-tls-capture)
        ENABLE_JAVA_TLS_CAPTURE="true"
        shift
        ;;
      --enable-tracepoints)
        ENABLE_TRACEPOINTS="true"
        shift
        ;;
      --no-download)
        DOWNLOAD_DISABLED="true"
        shift
        ;;
      --install-dir)
        NEW_INSTALLDIR="$2"
        RELOCATE_INSTALLDIR="true"
        shift
        shift
        ;;
      --log-dir)
        NEW_LOGDIR="$2"
        RELOCATE_LOGDIR="true"
        shift
        shift
        ;;
      -p|--package-url)
        package_url="$2"
        shift
        shift
        ;;
      --request-per-second-limit)
        EBPF_REQUEST_PER_SECOND_LIMIT="$2"
        shift
        shift
        ;;
      --max-connection)
        EBPF_MAX_CONNECTION="$2"
        shift
        shift
        ;;
      --go-memory-limit)
        EBPF_GO_MEMORY_LIMIT="$2"
        shift
        shift
        ;;
      --libssl-prefixes)
        EBPF_LIBSSL_PREFIXES="$2"
        shift
        shift
        ;;
      --override-config-file)
        EBPF_OVERRIDE_CONFIG_FILE="$2"
        shift
        shift
        ;;
      --no-verify)
        SIGNING_VERIFICATION="false"
        shift
        ;;
      help)
        print_help_ebpf_only
        ;;
      *) # unknown option
        printf "Unknown option %s.\n" "$key"
        error_msg "ebpf-only"
        ;;
    esac
  done
  cleanup_traceable_deployments
  check_input_empty_and_output_error "$SERVICE_NAME" "service name" "ebpf-only"
  check_input_empty_and_output_error "$TPA_ENDPOINT" "tpa endpoint" "ebpf-only"
  check_ebpf_loglevel "$EBPF_LOGLEVEL" "ebpf-only"
  check_mirror_mode "$MODE" "ebpf-only"
  check_no_download "ebpf-only"
  set_package_url "$package_url"

  if [ "$EXPORTER" != "ZIPKIN" ] && [ "$EXPORTER" != "OTLP" ]
  then
    printf "exporter must be one of: ZIPKIN, OTLP. Default value is OTLP. Received value is %s.\n" "$EXPORTER"
    error_msg "ebpf-only"
  fi
}

parse_input_http_proxy_only() {
  INSTALL_HTTP_PROXY="true"
  local package_url
  while [[ $# -gt 0 ]]
  do
    key="$1"
    case $key in
      -p|--port)
        PORT="$2"
        shift
        shift
        ;;
      --platform-host)
        PLATFORM_HOST="$2"
        shift
        shift
        ;;
      --platform-port)
        PLATFORM_PORT="$2"
        shift
        shift
        ;;
      --envoy-version)
        OVERRIDE_ENVOY_VERSION="$2"
        shift
        shift
        ;;
      --server-cert)
        SERVER_CERT="$2"
        shift
        shift
        ;;
      --server-key)
        SERVER_KEY="$2"
        shift
        shift
        ;;
      --proxy-host)
        PROXY_HOST="$2"
        shift
        shift
        ;;
      --proxy-port)
        PROXY_PORT="$2"
        shift
        shift
        ;;
      --package-url)
        package_url="$2"
        shift
        shift
        ;;
      --proxy-credentials-encoded)
        PROMPT_FOR_CREDENTIALS="true"
        shift
        ;;
      help)
        print_help_http_proxy_only
        ;;
      *) # unknown option
        printf "Unknown option: %s. \n" "$key"
        error_msg "http-proxy-only"
        ;;
    esac
  done

  set_package_url "$package_url"
}

parse_input_cleanup() {
  while [[ $# -gt 0 ]]
  do
    key="$1"
    case $key in
      help)
        print_help_cleanup
        ;;
      *) # unknown option
        printf "Unknown option %s.\n" "$key"
        error_msg "cleanup"
        ;;
    esac
  done
  cleanup_traceable_deployments
  exit
}

create_rpm_relocate_flags() {
  local installdir_flags
  local logdir_flags
  local datadir_flags
  if [ "$RELOCATE_INSTALLDIR" == "true" ]
  then
    installdir_flags="--relocate $(rpm --eval "%{_sysconfdir}")=$NEW_INSTALLDIR"
  fi
  if [ "$RELOCATE_LOGDIR" == "true" ]
  then
    logdir_flags="--relocate $(rpm --eval "%{_localstatedir}/traceable/log")=$NEW_LOGDIR"
    localstatedir_flags="--relocate /var=$NEW_LOGDIR"
  fi
  if [ "$RELOCATE_DATADIR" == "true" ]
  then
    datadir_flags="--relocate $(rpm --eval "%{_localstatedir}/traceable/persistence")=$NEW_DATADIR"
  fi

  TPA_RELOCATE_FLAGS="${installdir_flags} ${logdir_flags} ${datadir_flags}"
  EBPF_RELOCATE_FLAGS="${installdir_flags} ${logdir_flags}"
  MIRROR_RELOCATE_FLAGS="${installdir_flags}"
  PACKET_CAPTURE_RELOCATE_FLAGS="${localstatedir_flags}"
}

print_sysconfdir() {
  if [ "$RELOCATE_INSTALLDIR" == "true" ]
  then
    echo "$NEW_INSTALLDIR"
  else
    echo "/etc"
  fi
}

yum_setup() {
  create_rpm_relocate_flags

  if [ "$DOWNLOAD_DISABLED" == "true" ]
  then
    if [ "$INSTALL_TPA" == "true" ]
    then
      printf "Installing Traceable Platform Agent... "
      # shellcheck disable=SC2086
      rpm -i ${TPA_RELOCATE_FLAGS} "${ROOTDIR}/pkg/rpm/traceable-agent.${SERVICE_VERSION_RPM}.rpm" 1>/dev/null
      checkErrorAndExitWithMessage "Error while installing Traceable Platform Agent"
      echo "done."
    fi
    if [ "$INSTALL_EBPF" == "true" ]
    then
      printf "Installing traceable ebpf tracer... "
      # shellcheck disable=SC2086
      rpm -i ${EBPF_RELOCATE_FLAGS} "${ROOTDIR}/pkg/rpm/traceable-ebpf-tracer.${EBPF_TRACER_VERSION_RPM}.rpm" 1>/dev/null
      checkErrorAndExitWithMessage "Error while installing traceable-ebpf-tracer"
      echo "done."
    elif [ "$INSTALL_SURICATA" == "true" ]
    then
      printf "Installing suricata... "
      # shellcheck disable=SC2086
      rpm -i ${MIRROR_RELOCATE_FLAGS} "${ROOTDIR}/pkg/rpm/suricata.${SURICATA_VERSION_RPM}.rpm" 1>/dev/null
      checkErrorAndExitWithMessage "Error while installing suricata"
      echo "done."
      printf "Installing packet capture... "
      if [ "$ENABLE_TLS_CAPTURE" == "true" ]
      then
        # shellcheck disable=SC2086
        rpm -i ${PACKET_CAPTURE_RELOCATE_FLAGS} "${ROOTDIR}/pkg/rpm/tls-packet-capture.${TLS_PACKET_CAPTURE_VERSION_RPM}.rpm" 1>/dev/null
        checkErrorAndExitWithMessage "Error while installing tls-packet-capture"
      fi
      echo "done."
    fi
    return
  fi

  cat > /etc/yum.repos.d/traceable.repo << EndOfArtifactoryRepoMessage
[traceable]
name=traceable
baseurl=$PACKAGE_URL
enabled=1
EndOfArtifactoryRepoMessage
  if [ "$SIGNING_VERIFICATION" = "true" ]; then
    printf "gpgcheck=1\nrepo_gpgcheck=1\ngpgkey=https://packages.traceable.ai/public-key.asc\n"  >> /etc/yum.repos.d/traceable.repo
  else
    printf "gpgcheck=0\n" >> /etc/yum.repos.d/traceable.repo
  fi
  checkErrorAndExitWithMessage "Error while creating yum repo for traceable"
  if [ "$INSTALL_TPA" == "true" ]
  then
    printf "Installing Traceable Platform Agent... "
    yum --disablerepo="*" --enablerepo="traceable" install -q -y traceable-"$SERVICE_VERSION_RPM" 1>/dev/null
    checkErrorAndExitWithMessage "Error while installing Traceable Platform Agent"
    echo "done."
  fi
  if [ "$INSTALL_EBPF" == "true" ]
  then
    printf "Installing traceable ebpf tracer... "
    yum --disablerepo="*" --enablerepo="traceable" install -q -y traceable-ebpf-tracer-"$EBPF_TRACER_VERSION_RPM" 1>/dev/null
    checkErrorAndExitWithMessage "Error while installing traceable-ebpf-tracer"
    echo "done."
  elif [ "$INSTALL_SURICATA" == "true" ]
  then
    printf "Installing suricata... "
    yum --disablerepo="*" --enablerepo="traceable" install -q -y suricata-"$SURICATA_VERSION_RPM" 1>/dev/null
    checkErrorAndExitWithMessage "Error while installing suricata"
    echo "done."
    if [ "$ENABLE_TLS_CAPTURE" == "true" ]
      then
        yum --disablerepo="*" --enablerepo="traceable" install -q -y tls-packet-capture-"$TLS_PACKET_CAPTURE_VERSION_RPM" 1>/dev/null
        checkErrorAndExitWithMessage "Error while installing tls-packet-capture"
    fi
  fi
}

apt_setup(){
  if [ "$DOWNLOAD_DISABLED" == "true" ]
  then
    if [ "$INSTALL_TPA" == "true" ]
    then
      printf "Installing Traceable Platform Agent... "
      dpkg -i "${ROOTDIR}/pkg/deb/traceable-agent.${SERVICE_VERSION}.deb" 1>/dev/null
      checkErrorAndExitWithMessage "Error while installing Traceable Platform Agent"
      echo "done."
    fi
    if [ "$INSTALL_EBPF" == "true" ]
    then
      printf "Installing traceable ebpf tracer... "
      dpkg -i "${ROOTDIR}/pkg/deb/traceable-ebpf-tracer.${EBPF_TRACER_VERSION}.deb" 1>/dev/null
      checkErrorAndExitWithMessage "Error while installing traceable-ebpf-tracer"
      echo "done."
    elif [ "$INSTALL_SURICATA" == "true" ]
    then
      printf "Installing suricata... "
      dpkg -i "${ROOTDIR}/pkg/deb/suricata.${SURICATA_VERSION}.deb" 1>/dev/null
      checkErrorAndExitWithMessage "Error while installing suricata"
      echo "done."
      if [ "$ENABLE_TLS_CAPTURE" == "true" ]
      then
        dpkg -i "${ROOTDIR}/pkg/deb/tls-packet-capture.${TLS_PACKET_CAPTURE_VERSION}.deb" 1>/dev/null
        checkErrorAndExitWithMessage "Error while installing tls-packet-capture"
      fi
    fi
    return
  fi
  if [ "$SIGNING_VERIFICATION" = "true" ]; then
    curl https://packages.traceable.ai/public-key.asc 2>/dev/null | gpg --dearmor -o /usr/share/keyrings/traceable.gpg
    echo "deb [signed-by=/usr/share/keyrings/traceable.gpg] $PACKAGE_URL $VERSION_CODENAME_UBUNTU main" > /etc/apt/sources.list.d/traceable.list
  else
    echo "deb [trusted=yes] $PACKAGE_URL $VERSION_CODENAME_UBUNTU main" > /etc/apt/sources.list.d/traceable.list
  fi
  cat > /etc/apt/preferences.d/traceable << EndOfTraceableListMessage
Package: *
Pin: release o=Traceable
Pin-Priority: 900
EndOfTraceableListMessage
  checkErrorAndExitWithMessage "Error while creating apt repo for traceable"
  apt-get update -y 1>/dev/null
  checkErrorAndExitWithMessage "Error while updating apt with traceable repo"
  if [ "$INSTALL_TPA" == "true" ]
  then
    printf "Installing Traceable Platform Agent... "
    apt-get install -y traceable="${SERVICE_VERSION}-$VERSION_CODENAME_UBUNTU" --allow-downgrades 1>/dev/null
    checkErrorAndExitWithMessage "Error while installing Traceable Platform Agent"
    echo "done."
  fi
  if [ "$INSTALL_EBPF" == "true" ]
  then
    printf "Installing traceable ebpf tracer... "
    apt-get install -y traceable-ebpf-tracer="${EBPF_TRACER_VERSION}" --allow-downgrades 1>/dev/null
    checkErrorAndExitWithMessage "Error while installing traceable-ebpf-tracer"
    echo "done."
  elif [ "$INSTALL_SURICATA" == "true" ]
  then
    printf "Installing suricata... "
    apt-get install -y suricata="${SURICATA_VERSION}" --allow-downgrades 1>/dev/null
    checkErrorAndExitWithMessage "Error while installing suricata"
    echo "done."
    if [ "$ENABLE_TLS_CAPTURE" == "true" ]
      then
        apt-get install -y tls-packet-capture="${TLS_PACKET_CAPTURE_VERSION}" --allow-downgrades 1>/dev/null
        checkErrorAndExitWithMessage "Error while installing tls-packet-capture"
    fi
  fi
}

create_suricata_service() {
  MY_IP=$(hostname -I 2>/dev/null | cut -d ' ' -f 1)
  if [ "$ENABLE_CUSTOM_FILTER" == "false" ] && [ -n "$MY_IP" ] && [ "$ENABLE_REMOTE_INSPECTION_SURICATA" == "false" ]
  then
    FILTER_OPTION="not host $MY_IP"
  fi

  if [ "$MIRROR_MAX_MEM" != "" ]
  then
    MIRROR_MEMORY_LIMIT_COMMAND="${MEMORY_CONFIGURATION_KEY}=${MIRROR_MAX_MEM}"
    if [ "$SET_MEMORY_ACCOUNTING" == "true" ]
    then
      MIRROR_MEMORY_ACCOUNTING_COMMAND="$MEMORY_ACCOUNTING_COMMAND"
    fi
  fi

  FLAGS="-c $(print_sysconfdir)/suricata/suricata.yaml"
  if [ "$RELOCATE_LOGDIR" == "true" ]
  then
    FLAGS="$FLAGS -l $NEW_LOGDIR"
  fi
  if [ "$ENABLE_REMOTE_INSPECTION_SURICATA" == "true" ]
  then
    FLAGS="$FLAGS --set stream.allow-gaps=true"
  fi

  cat > ${MIRROR_SERVICE_FILE} << EndOfSuricataServiceMessage
[Unit]
Description=Suricata Intrusion Detection Service
After=$MIRROR_AFTER_TARGET

[Service]
ExecStartPre=/bin/rm -f /var/run/suricata.pid
ExecStart=/usr/bin/suricata $FLAGS --pidfile /var/run/suricata.pid $1 $FILTER_OPTION
ExecReload=/bin/kill -USR2 \$MAINPID
Restart=always
RestartSec=3
$MIRROR_MEMORY_LIMIT_COMMAND
$MIRROR_MEMORY_ACCOUNTING_COMMAND

[Install]
WantedBy=multi-user.target

EndOfSuricataServiceMessage
}

setup_dummy_interface() {
    msg=$(ip link add mirror0 type dummy 2>&1)
    error=$?
    if [ $error -ne 0 ] && ! [[ "$msg" =~ "exists" ]]
    then
        echo "Error while creating dummy interface: $msg."
        exit 1
    fi
    ifconfig mirror0 up
    checkErrorAndExitWithMessage "Error while changing interface mirror0 to up"
    ifconfig mirror0 mtu 65535
    checkErrorAndExitWithMessage "Error while setting mtu of interface mirror0"
}

create_tls_packet_capture_service() {
  MY_IP=$(hostname -I 2>/dev/null | cut -d ' ' -f 1)
  if [ "$ENABLE_CUSTOM_FILTER" == "false" ] && [ -n "$MY_IP" ]
  then
    FILTER_OPTION="not host $MY_IP"
  fi

  cat > /usr/bin/tls-packet-capture-wrapper << EndOfTLSWrapperMessage
#!/bin/bash
ip link add mirror0 type dummy
ifconfig mirror0 up
ifconfig mirror0 mtu 65535
/usr/bin/tls-packet-capture \$@
EndOfTLSWrapperMessage
  chmod +x /usr/bin/tls-packet-capture-wrapper

  if [ "$PACKET_CAPTURE_MAX_MEM" != "" ]
  then
    PC_MEMORY_LIMIT_COMMAND="${MEMORY_CONFIGURATION_KEY}=${PACKET_CAPTURE_MAX_MEM}"
    if [ "$SET_MEMORY_ACCOUNTING" == "true" ]
    then
      PC_MEMORY_ACCOUNTING_COMMAND="$MEMORY_ACCOUNTING_COMMAND"
    fi
  fi

  cat > ${PACKET_CAPTURE_SERVICE_FILE} << EndOfTLSServiceMessage
[Unit]
Description=TLS Packet Capture Service
After=$PACKET_CAPTURE_AFTER_TARGET

[Service]
ExecStart=/usr/bin/tls-packet-capture-wrapper $TLS_PACKET_CAPTURE_INTERFACE_OPTIONS $TLS_PACKET_CAPTURE_TLS_OPTIONS -D 30 -w i:mirror0 -L $SERVER_PORT $FILTER_OPTION
ExecReload=/bin/kill -USR2 \$MAINPID
Restart=always
RestartSec=3
$PC_MEMORY_LIMIT_COMMAND
$PC_MEMORY_ACCOUNTING_COMMAND

[Install]
WantedBy=multi-user.target

EndOfTLSServiceMessage
}

update_traceable_agent_service() {
  if [ "$REPLACE_TPA_AFTER_TARGET" == "true" ]
  then
    sed -i "s/^After=.*$/After=${TPA_AFTER_TARGET}/" ${TPA_SERVICE_FILE}
  fi

  mkdir -p "${TPA_SERVICE_DIR}"
  if [ "$TPA_MAX_MEM" != "" ]
  then
    cat > "${TPA_SERVICE_DIR}/memory-limit.conf" << EndOfTraceableMemoryMessage
[Service]
${MEMORY_CONFIGURATION_KEY}=${TPA_MAX_MEM}
EndOfTraceableMemoryMessage
    if [ "$SET_MEMORY_ACCOUNTING" == "true" ]
    then
      cat > "${TPA_SERVICE_DIR}/memory-accounting.conf" << EndOfTraceableMemoryAccountingMessage
[Service]
$MEMORY_ACCOUNTING_COMMAND
EndOfTraceableMemoryAccountingMessage
    fi
  fi

  if [ "$TPA_MAX_CPU" != "" ]
    then
      cat > "${TPA_SERVICE_DIR}/cpu-limit.conf" << EndOfTraceableCpuMessage
[Service]
${CPU_CONFIGURATION_KEY}=${TPA_MAX_CPU}
EndOfTraceableCpuMessage
      if [ "$SET_CPU_ACCOUNTING" == "true" ]
      then
        cat > "${TPA_SERVICE_DIR}/cpu-accounting.conf" << EndOfTraceableCpuAccountingMessage
[Service]
$CPU_ACCOUNTING_COMMAND
EndOfTraceableCpuAccountingMessage
    fi
  fi
}

update_ebpf_service() {
  if [ "$REPLACE_EBPF_AFTER_TARGET" == "true" ]
  then
    sed -i "s/^After=.*$/After=${EBPF_AFTER_TARGET}/" ${EBPF_SERVICE_FILE}
  fi

  mkdir -p "${EBPF_SERVICE_DIR}"
  if [ "$EBPF_MAX_MEM" != "" ]
  then
    cat > "${EBPF_SERVICE_DIR}/memory-limit.conf" << EndOfEbpfMemoryMessage
[Service]
${MEMORY_CONFIGURATION_KEY}=${EBPF_MAX_MEM}
EndOfEbpfMemoryMessage
    if [ "$SET_MEMORY_ACCOUNTING" == "true" ]
    then
      cat > "${EBPF_SERVICE_DIR}/memory-accounting.conf" << EndOfEbpfMemoryAccountingMessage
[Service]
$MEMORY_ACCOUNTING_COMMAND
EndOfEbpfMemoryAccountingMessage
    fi
  fi

  if [ "$EBPF_MAX_CPU" != "" ]
    then
      cat > "${EBPF_SERVICE_DIR}/cpu-limit.conf" << EndOfEbpfCpuMessage
[Service]
${CPU_CONFIGURATION_KEY}=${EBPF_MAX_CPU}
EndOfEbpfCpuMessage
      if [ "$SET_CPU_ACCOUNTING" == "true" ]
      then
        cat > "${EBPF_SERVICE_DIR}/cpu-accounting.conf" << EndOfEbpfCpuAccountingMessage
[Service]
$CPU_ACCOUNTING_COMMAND
EndOfEbpfCpuAccountingMessage
    fi
  fi
}

update_traceable_environment() {
  mkdir -p "${TPA_SERVICE_DIR}"
  cat > "${TPA_SERVICE_DIR}/environment.conf" << EndOfTraceableEnvMessage
[Service]
Environment="TA_ENVIRONMENT=$ENVIRONMENT"
EndOfTraceableEnvMessage

if [ -n "$HTTP_PROXY" ]; then
  echo "Environment=\"HTTP_PROXY=$HTTP_PROXY\"" >> "${TPA_SERVICE_DIR}/environment.conf"
fi
if [ -n "$HTTPS_PROXY" ]; then
  echo "Environment=\"HTTPS_PROXY=$HTTPS_PROXY\"" >> "${TPA_SERVICE_DIR}/environment.conf"
fi
if [ -n "$NO_PROXY" ]; then
  echo "Environment=\"NO_PROXY=$NO_PROXY\"" >> "${TPA_SERVICE_DIR}/environment.conf"
fi

}

update_tpa_remote_endpoint() {
  if [ -n "$REMOTE_ENDPOINT" ]
  then
    export NAME="${REMOTE_ENDPOINT}:443"
    $YQ_BIN -i '.global.remote.endpoint = strenv(NAME)' "$1"
  fi
}
update_tpa_remote_cert_path() {
  if [ -n "$REMOTE_CERT_PATH" ]
  then
    export VAR="$REMOTE_CERT_PATH"
    $YQ_BIN -i '.global.remote.ca_cert_file = strenv(VAR) | .global.remote.ca_cert_file style="double"' "$1"
  fi
}
update_tpa_log_level() {
  if [ -n "$TPA_LOGLEVEL" ]
  then
    export VAR="$TPA_LOGLEVEL"
    $YQ_BIN -i '.global.logging.level = strenv(VAR)' "$1"
  fi
}
update_tpa_log_max_size() {
  if [ -n "$TPA_LOG_MAX_SIZE_MB" ]
  then
    $YQ_BIN -i ".global.log_rotation.max_size = $TPA_LOG_MAX_SIZE_MB" "$1"
  fi
}
update_tpa_log_max_backups() {
  if [ -n "$TPA_LOG_MAX_BACKUPS" ]
  then
    $YQ_BIN -i ".global.log_rotation.max_backups = $TPA_LOG_MAX_BACKUPS" "$1"
  fi
}
update_tpa_service_name() {
  export VAR="$SERVICE_NAME"
  $YQ_BIN -i '.ext_cap.service_name = strenv(VAR)' "$1"
}
update_tpa_collector_otlp_storage_extension_directory() {
  if [ -n "$OTLP_FILE_STORAGE_DIR" ]
  then
    export VAR="$OTLP_FILE_STORAGE_DIR"
    $YQ_BIN -i ".collector.config.extensions.file_storage.directory = strenv(VAR)" "$1"
    $YQ_BIN -i ".collector.config.extensions.file_storage.compaction.directory = strenv(VAR)" "$1"
  elif [ "$RELOCATE_DATADIR" == "true" ]
  then
    export VAR="$NEW_DATADIR"
    $YQ_BIN -i ".collector.config.extensions.file_storage.directory = strenv(VAR)" "$1"
    $YQ_BIN -i ".collector.config.extensions.file_storage.compaction.directory = strenv(VAR)" "$1"
  fi
}
update_tpa_collector_otlp_exporter_sending_queue_size() {
  if [ -n "$OTLP_EXPORTER_SENDING_QUEUE_SIZE" ]
  then
    $YQ_BIN -i ".collector.config.exporters.otlp.sending_queue.queue_size = $OTLP_EXPORTER_SENDING_QUEUE_SIZE" "$1"
  fi
}
update_tpa_tls_server() {
  if [ -n "$TPA_TLS_MODE" ]
  then
    export VAR="$TLS_ENDPOINT"
    $YQ_BIN -i '.global.tls_server.endpoint = strenv(VAR) | .global.tls_server.endpoint style="double"' "$1"
    export VAR="$TLS_KEY_FILE"
    $YQ_BIN -i '.global.tls_server.key_file = strenv(VAR) | .global.tls_server.key_file style="double"' "$1"
    export VAR="$TLS_CERT_FILE"
    $YQ_BIN -i '.global.tls_server.cert_file = strenv(VAR) | .global.tls_server.cert_file style="double"' "$1"
    export VAR="$TLS_ROOT_CERT_FILE"
    $YQ_BIN -i '.global.tls_server.root_cert_file = strenv(VAR) | .global.tls_server.root_cert_file style="double"' "$1"
  fi
}
update_tpa_trace_propagation_format() {
  if [ -n "$TPA_TRACE_PROPAGATION_FORMAT" ]
  then
    export VAR="$TPA_TRACE_PROPAGATION_FORMAT"
    # this would match every element in the array
    $YQ_BIN -i 'del(.global.telemetry.propagation_formats[])' "$1"
    $YQ_BIN -i '.global.telemetry.propagation_formats[0] = strenv(VAR)' "$1"
  fi
}
update_ext_cap_mirror_sock_address() {
  export VAR="/var/log/suricata/eve.json"
  if [ "$RELOCATE_LOGDIR" == "true" ]
  then
    export VAR="$NEW_LOGDIR/eve.json"
  fi
  $YQ_BIN -i '.ext_cap.mirror.sock_addr = strenv(VAR) | .ext_cap.mirror.sock_addr style="double"' "$1"
}
# disable ext_cap blocking unless --ext-cap-enable-blocking flag is provided.
update_ext_cap_blocking_config() {
  $YQ_BIN -i ".ext_cap.blocking_config.enabled = ${TPA_EXT_CAP_ENABLE_BLOCKING}" "$1"
}
update_tpa_collector_otlp_max_connection_age() {
  if [ -n "$OTLP_MAX_CONNECTION_AGE" ]
  then
    export VAR="$OTLP_MAX_CONNECTION_AGE"
    $YQ_BIN -i ".collector.config.receivers.otlp.protocols.grpc.keepalive.server_parameters.max_connection_age = strenv(VAR)" "$1"
  fi
}
update_tpa_hsl_config(){
  if [ -n "$HSL_ENABLED" ]
  then
    $YQ_BIN -i ".global.hsl_server.enabled = true" "$1"
  fi
  if [ -n "$HSL_DELIMITER" ]
  then
    export VAR="$HSL_DELIMITER"
    $YQ_BIN -i ".global.hsl_server.delimiter = strenv(VAR)" "$1"
  fi
  if [ -n "$HSL_ENDPOINT" ]
  then
    export VAR="$HSL_ENDPOINT"
    $YQ_BIN -i ".global.hsl_server.server.endpoint = strenv(VAR)" "$1"
  fi
  if [ -n "$HSL_TLS_CERT" ]
  then
    export VAR="$HSL_TLS_CERT"
    $YQ_BIN -i ".global.hsl_server.server.cert_file = strenv(VAR)" "$1"
  fi
  if [ -n "$HSL_TLS_KEY" ]
  then
    export VAR="$HSL_TLS_KEY"
    $YQ_BIN -i ".global.hsl_server.server.key_file = strenv(VAR)" "$1"
  fi
  if [ -n "$HSL_MAX_QUEUE_SIZE" ]
  then
    $YQ_BIN -i ".global.hsl_server.server.max_queue_size = $HSL_MAX_QUEUE_SIZE" "$1"
  fi
  if [ -n "$HSL_BUFFER_SIZE" ]
  then
    $YQ_BIN -i ".global.hsl_server.server.buffer_size = $HSL_BUFFER_SIZE" "$1"
  fi
}

update_tpa_using_override_config_file() {
  if [ -n "$TPA_OVERRIDE_CONFIG_FILE" ]
  then
    mkdir -p "${TPA_SERVICE_DIR}"
    cat > "${TPA_SERVICE_DIR}/override_config_file.conf" << EndOfTraceableOverrideConfigFile
[Service]
Environment="TA_CONFIG_OVERRIDE_FILE=$TPA_OVERRIDE_CONFIG_FILE"
EndOfTraceableOverrideConfigFile
  fi
}

update_traceable_agent_config() {
  # shellcheck disable=SC2155
  local configpath="$(print_sysconfdir)/traceable/agent/config.yaml"
  update_tpa_using_override_config_file "${configpath}"
  # The values in the override config file will be applied over the cmdline arg options.
  # Since it is applied using the TA_CONFIG_OVERRIDE_FILE env var it will be merged into
  # the config during runtime.
  update_tpa_remote_endpoint "${configpath}"
  update_tpa_remote_cert_path "${configpath}"
  update_tpa_log_level "${configpath}"
  update_tpa_log_max_size "${configpath}"
  update_tpa_log_max_backups "${configpath}"
  update_tpa_service_name "${configpath}"
  update_tpa_collector_otlp_storage_extension_directory "${configpath}"
  update_tpa_collector_otlp_exporter_sending_queue_size "${configpath}"
  update_ext_cap_blocking_config "${configpath}"
  update_tpa_trace_propagation_format "${configpath}"
  update_tpa_collector_otlp_max_connection_age "${configpath}"
  update_tpa_hsl_config "$configpath"

  if [ "$UPDATE_AGENT_CONFIG_WITH_GRPC_TO_HTTP_PROXY" = "true" ]; then
      echo "Updating TPA to use local envoy grpc to http proxy"
      export G2H_TARGET_ENDPOINT="127.0.0.1:${G2H_PORT}"
      $YQ_BIN -i ".global.remote.endpoint = strenv(G2H_TARGET_ENDPOINT)" "${configpath}"
      $YQ_BIN -i ".global.remote.secure = env(G2H_TPA_CONFIG_SECURE)" "${configpath}"
  fi

  $YQ_BIN -i '.ext_cap.enabled = true' "${configpath}"
  $YQ_BIN -i '.ext_cap.sampling.enabled = true' "${configpath}"

  update_tpa_tls_server "${configpath}"

  [ "$INSTALL_SURICATA" == "false" ] && return

  $YQ_BIN -i '.ext_cap.mirror.enabled = true' "${configpath}"
  $YQ_BIN -i '.ext_cap.mirror.max_buffer_size = 524288' "${configpath}"
  $YQ_BIN -i '.ext_cap.mirror.io_timeout = 60' "${configpath}"
  $YQ_BIN -i '.ext_cap.mirror.background_stats_wait = 300' "${configpath}"
  $YQ_BIN -i '.ext_cap.mirror.max_queue_depth = 5000' "${configpath}"
  update_ext_cap_mirror_sock_address "${configpath}"
}

update_ebpf_service_name() {
  export VAR="$SERVICE_NAME"
  $YQ_BIN -i '.service_name = strenv(VAR) | .service_name style="double"' "$1"
}
update_ebpf_port_list() {
  if [ -n "$PORTS" ]
  then
    $YQ_BIN -i '.port_list = "" | .port_list tag="!!null"' "$1"
    ports=$(echo "$PORTS" | tr "," "\n")
    for port in $ports
    do
      $YQ_BIN -i ".port_list += [$port]" "$1"
    done
  fi
}
update_ebpf_ssl_dirs() {
  if [ -n "$SSL_DIRS" ]
  then
    $YQ_BIN -i '.ssl_dirs = "" | .ssl_dirs tag="!!null"' "$1"
    dir_list=$(echo "$SSL_DIRS" | tr "," "\n")
    for dir in $dir_list
    do
      export VAR="$dir"
      $YQ_BIN -i '.ssl_dirs += [strenv(VAR)] | .ssl_dirs.[] style="double"' "$1"
    done
  fi
}
update_ebpf_exclude_processes() {
  if [ -n "$EXCLUDE_PROCESSES" ]
  then
    $YQ_BIN -i '.exclude_processes = "" | .exclude_processes tag="!!null"' "$1"
    proc_list=$(echo "$EXCLUDE_PROCESSES" | tr "," "\n")
    for proc in $proc_list
    do
      export VAR="$proc"
      $YQ_BIN -i '.exclude_processes += [strenv(VAR)] | .exclude_processes.[] style="double"' "$1"
    done
  fi
}
update_ebpf_max_return_probes() {
  if [ -n "$MAX_RETURN_PROBES" ]
  then
    $YQ_BIN -i ".max_active_ret_probe = $MAX_RETURN_PROBES" "$1"
  fi
}
update_ebpf_mode() {
  if [ -n "$MODE" ]
  then
    export VAR="$MODE"
    $YQ_BIN -i '.mode = strenv(VAR) | .mode style="double"' "$1"
  fi
}
update_ebpf_endpoints() {
  if [ -n "$TPA_ENDPOINT" ]
  then
    local reporting_path
    local scheme
    local reporting_port
    local remote_port
    if [ "$EXPORTER" == "ZIPKIN" ]
    then
      reporting_path="/api/v2/spans"
      ## set scheme only for ZIPKIN
      if [ -n "$CA_CERT_PATH" ]
      then
        scheme="https://"
      else
        scheme="http://"
      fi
    fi

    # set default ports based on the scheme
    if [ -n "$CA_CERT_PATH" ]
    then
      reporting_port="$REPORTING_PORT_TLS_DEFAULT"
      remote_port="$REMOTE_PORT_TLS_DEFAULT"
    else
      reporting_port="$REPORTING_PORT_DEFAULT"
      remote_port="$REMOTE_PORT_DEFAULT"
    fi

    # update ports based on command-line arguments
    if [ -n "$REPORTING_PORT" ]
    then
      reporting_port="$REPORTING_PORT"
    fi
    if [ -n "$REMOTE_PORT" ]
    then
      remote_port="$REMOTE_PORT"
    fi

    export VAR="${scheme}${TPA_ENDPOINT}:${reporting_port}${reporting_path}"
    $YQ_BIN -i '.agent_config.reporting.endpoint = strenv(VAR) | .agent_config.reporting.endpoint style="double"' "$1"

    export VAR="${TPA_ENDPOINT}:${remote_port}"
    $YQ_BIN -i '.agent_config.remote.endpoint = strenv(VAR) | .agent_config.remote.endpoint style="double"' "$1"
  fi
}
update_ebpf_ca_cert_file() {
  if [ -n "$CA_CERT_PATH" ]
  then
    $YQ_BIN -i '.agent_config.secure = true' "$1"
    export VAR="$CA_CERT_PATH"
    $YQ_BIN -i '.agent_config.ca_cert_file = strenv(VAR) | .agent_config.ca_cert_file style="double"' "$1"
  fi
}
update_ebpf_log_level() {
  if [ -n "$EBPF_LOGLEVEL" ]
  then
    export VAR="$EBPF_LOGLEVEL"
    $YQ_BIN -i '.logging.level = strenv(VAR)' "$1"
  fi
}
update_ebpf_log_max_size() {
  if [ -n "$EBPF_LOG_MAX_SIZE_MB" ]
  then
    $YQ_BIN -i ".logging.log_rotation.max_size = $EBPF_LOG_MAX_SIZE_MB" "$1"
  fi
}
update_ebpf_log_max_backups() {
  if [ -n "$EBPF_LOG_MAX_BACKUPS" ]
  then
    $YQ_BIN -i ".logging.log_rotation.max_backups = $EBPF_LOG_MAX_BACKUPS" "$1"
  fi
}
update_ebpf_java_tls_capture() {
  if [ "$ENABLE_JAVA_TLS_CAPTURE" == "true" ]
  then
    $YQ_BIN -i ".enable_java_tls_capture = true" "$1"
  fi
}
update_ebpf_tracepoints() {
  if [ "$ENABLE_TRACEPOINTS" == "true" ]
  then
    $YQ_BIN -i ".enable_tracepoints = true" "$1"
  fi
}
update_ebpf_requests_per_second_limit() {
  if [ -n "$EBPF_REQUEST_PER_SECOND_LIMIT" ]
  then
      $YQ_BIN -i ".request_per_second_limit = $EBPF_REQUEST_PER_SECOND_LIMIT" "$1"
  fi
}
update_ebpf_max_connection() {
  if [ -n "$EBPF_MAX_CONNECTION" ]
  then
      $YQ_BIN -i ".max_connection = $EBPF_MAX_CONNECTION" "$1"
  fi
}
update_ebpf_go_memory_limit() {
  if [ -n "$EBPF_GO_MEMORY_LIMIT" ]
  then
    export VAR="$EBPF_GO_MEMORY_LIMIT"
    $YQ_BIN -i '.go_memory_limit = strenv(VAR) | .go_memory_limit style="double"' "$1"
  fi
}
update_ebpf_libssl_prefixes() {
  if [ -n "$EBPF_LIBSSL_PREFIXES" ]
  then
    $YQ_BIN -i '.libssl_prefixes = "" | .libssl_prefixes tag="!!null"' "$1"
    prefixes=$(echo "$EBPF_LIBSSL_PREFIXES" | tr "," "\n")
    for prefix in $prefixes
    do
      $YQ_BIN -i ".libssl_prefixes += [$prefix]" "$1"
    done
  fi
}
update_ebpf_using_override_config_file() {
  if [ -n "$EBPF_OVERRIDE_CONFIG_FILE" ]
  then
    $YQ_BIN -i '. *= load("'$EBPF_OVERRIDE_CONFIG_FILE'")' "$1"
  fi
}
update_ebpf_config() {
  # shellcheck disable=SC2155
  local configpath="$(print_sysconfdir)/traceable/ebpf-tracer/config.yaml"
  update_ebpf_using_override_config_file "${configpath}"
  # The following configs will override what was in the override file.
  update_ebpf_service_name "${configpath}"
  update_ebpf_port_list "${configpath}"
  update_ebpf_ssl_dirs "${configpath}"
  update_ebpf_exclude_processes "${configpath}"
  update_ebpf_max_return_probes "${configpath}"
  update_ebpf_mode "${configpath}"
  update_ebpf_endpoints "${configpath}"
  update_ebpf_ca_cert_file "${configpath}"
  update_ebpf_log_level "${configpath}"
  update_ebpf_log_max_size "${configpath}"
  update_ebpf_log_max_backups "${configpath}"
  update_ebpf_java_tls_capture "${configpath}"
  update_ebpf_tracepoints "${configpath}"
  update_ebpf_requests_per_second_limit "${configpath}"
  update_ebpf_max_connection "${configpath}"
  update_ebpf_go_memory_limit "${configpath}"
  update_ebpf_libssl_prefixes "${configpath}"
}

read_refresh_token() {
  echo -n "Traceable Access Token: "
  local token
  stty -echo
  while IFS= read -p "$prompt" -r -s -n 1 char
  do
    if [[ $char == $'\0' ]]
    then
      break
    fi
    if [[ $char == $'\177' ]] ; then
      if [ "$CHARCOUNT" -gt 0 ] ; then
        CHARCOUNT=$((CHARCOUNT-1))
        prompt=$'\b \b'
        token="${token%?}"
      else
        prompt=''
      fi
    else
      CHARCOUNT=$((CHARCOUNT+1))
      prompt='*'
      token+="$char"
    fi
  done
  stty echo
  if [ -z "$token" ]
  then
    echo "Token can not be empty."
    exit
  fi
  REFRESH_TOKEN=$token
  echo "" # add new line after input
}

read_proxy_credentials_encoded_token() {
  echo -n "Proxy Auth credentials: "
  local token
  stty -echo
  while IFS= read -p "$prompt" -r -s -n 1 char
  do
    if [[ $char == $'\0' ]]
    then
      break
    fi
    if [[ $char == $'\177' ]] ; then
      if [ "$CHARCOUNT" -gt 0 ] ; then
        CHARCOUNT=$((CHARCOUNT-1))
        prompt=$'\b \b'
        token="${token%?}"
      else
        prompt=''
      fi
    else
      CHARCOUNT=$((CHARCOUNT+1))
      prompt='*'
      token+="$char"
    fi
  done
  stty echo
  if [ -z "$token" ]
  then
    echo "Credentials can not be empty."
    exit
  fi
  PROXY_CREDENTIALS=$token
  echo "" # add new line after input
}

update_traceable_token() {
  if [ -z "$REFRESH_TOKEN" ]
  then
    return
  fi

  echo "Updating token"
  cat > "$(print_sysconfdir)/traceable/agent/token" << EndOfTokenMessage
$REFRESH_TOKEN
EndOfTokenMessage
}

check_cpu_accounting() {
  if [ "$(systemctl show -p DefaultCPUAccounting | cut -d'=' -f2)" == "no" ]
  then
    SET_CPU_ACCOUNTING="true"
  fi
}

decode_tpa_certs() {
  mkdir -p /etc/traceable/agent/certs/
  echo "$TLS_KEY_BASE64" | base64 -d > /etc/traceable/agent/certs/key.pem
  checkErrorAndExitWithMessage "Error while base64 decoding TLS key"
  echo "$TLS_CERT_BASE64" | base64 -d > /etc/traceable/agent/certs/cert.pem
  checkErrorAndExitWithMessage "Error while base64 decoding TLS certificate"
  echo "$TLS_ROOT_CERT_BASE64" | base64 -d > /etc/traceable/agent/certs/cacert.pem
  checkErrorAndExitWithMessage "Error while base64 decoding TLS ca cert"

  TLS_KEY_FILE="/etc/traceable/agent/certs/key.pem"
  TLS_CERT_FILE="/etc/traceable/agent/certs/cert.pem"
  TLS_ROOT_CERT_FILE="/etc/traceable/agent/certs/cacert.pem"

}

set_distribution_id() {
  local id
  local version_id
  id=$(grep '^ID=' /etc/os-release | cut -d '=' -f 2 | cut -d '"' -f 2)
  version_id=$(grep '^VERSION_ID=' /etc/os-release | cut -d '=' -f 2 | cut -d '"' -f 2)
  if [ "$id" == "ubuntu" ]
  then
    VERSION_CODENAME_UBUNTU=$(grep '^VERSION_CODENAME=' /etc/os-release | cut -d '=' -f 2 | cut -d '"' -f 2)
     # we only publish packages for focal(20.04) and bionic(18.04) among the ubuntu LTS, overriding to focal in any higher version
    if [[ $version_id =~ ^2[2-9]\.[0-9][0-9] ]]
    then
      VERSION_CODENAME_UBUNTU="focal"
    fi
  fi
}

set_memory_configuration_key() {
  if [ -f "/sys/fs/cgroup/cgroup.controllers" ]
  then
    MEMORY_CONFIGURATION_KEY="MemoryMax"
  else
    MEMORY_CONFIGURATION_KEY="MemoryLimit"
  fi
}

install_dep() {
  if [ "$DOWNLOAD_DISABLED" == "true" ]
  then
    YQ_BIN="${ROOTDIR}/bin/yq"
    return
  fi
  if [ "$INSTALL_SURICATA" == "true" ] && [ "$ENABLE_TLS_CAPTURE" == "true" ]
  then
    # shellcheck disable=SC2155
    local ifconfig_status=$(which ifconfig)
    local install_net_tools=""
    if [ -z "$ifconfig_status" ]
    then
        install_net_tools="net-tools"
    fi
  fi
  local architecture
  local arch_name
  architecture=$(uname -m)
  if [ "$architecture" == "aarch64" ] || [ "$architecture" == "arm64" ]
  then
    arch_name="arm64"
  elif [ "$architecture" == "x86_64" ] || [ "$architecture" == "amd64" ]
  then
    arch_name="amd64"
  else
    arch_name="386"
  fi

  if [ "$PACKAGE_MANAGER" == "apt" ]
  then
    apt-get install -y wget ca-certificates $install_net_tools 1>/dev/null
  else
    yum install -y wget ca-certificates $install_net_tools 1>/dev/null
  fi

  YQ_BIN="${SCRIPTDIR}/yq"
  YQ_VERSION="4.44.5"
  if command -v "$YQ_BIN" >/dev/null 2>&1
  then
    if [ "$($YQ_BIN --version 2>&1 | grep -c $YQ_VERSION)" -eq "1" ]
    then
      return
    fi
    echo -n "yq found at $YQ_BIN but version is not $YQ_VERSION. Reinstalling yq... "
    mv "$YQ_BIN" "${YQ_BIN}.bk"
  else
    printf "Installing yq... "
  fi

  wget -q "https://github.com/mikefarah/yq/releases/download/v${YQ_VERSION}/yq_linux_${arch_name}" -O "$YQ_BIN"
  chmod +x "$YQ_BIN"
  $YQ_BIN --version
  checkErrorAndExitWithMessage "Error in downloading yq v${YQ_VERSION}."
  echo "done."
}

setup_selinux_ebpf() {
  if [ "$PACKAGE_MANAGER" != "yum" ] || ! selinuxenabled
  then
    return
  fi

  printf "Setting up SELinux permissions... "
  local traceable_selinux_policy_location
  if [ "$DOWNLOAD_DISABLED" == "true" ]
  then
    traceable_selinux_policy_location="${ROOTDIR}/resources/traceable-ebpf-tracer.pp"
  else
    yum install -y policycoreutils-devel >/dev/null
    cat > ./traceable-ebpf-tracer.te << EndOfSELinuxMessage
module traceable-ebpf-tracer 1.0;

require {
        type unconfined_service_t;
        class bpf { map_create map_read map_write prog_load prog_run };
}

#============= unconfined_service_t ==============
allow unconfined_service_t self:bpf { map_create map_read map_write prog_load prog_run };

EndOfSELinuxMessage
    make -f /usr/share/selinux/devel/Makefile traceable-ebpf-tracer.pp >/dev/null
    traceable_selinux_policy_location="traceable-ebpf-tracer.pp"
  fi
  semodule -i "${traceable_selinux_policy_location}" >/dev/null
  checkErrorAndExitWithMessage "Error while installing SELinux policy ${traceable_selinux_policy_location}."
  echo "done."
}

install_http_proxy() {
  if [ "$PROMPT_FOR_CREDENTIALS" == "true" ]
    then
      read_proxy_credentials_encoded_token
    fi
  if [ -n "$OVERRIDE_ENVOY_VERSION" ]; then
      ENVOY_VERSION="$OVERRIDE_ENVOY_VERSION"
  else
      ENVOY_VERSION="1.30.2"
  fi

  export G2H_PORT=${PORT:-80}
  export G2H_PLATFORM_HOST=$PLATFORM_HOST
  export G2H_PLATFORM_PORT=${PLATFORM_PORT:-443}
  export G2H_PROXY_HOST=$PROXY_HOST
  export G2H_PROXY_PORT=${PROXY_PORT:-8080}
  export G2H_TPA_CONFIG_SECURE="false"
  export G2H_SERVER_CERT=$SERVER_CERT
  export G2H_SERVER_KEY=$SERVER_KEY
  export G2H_PLATFORM_HOST_WITH_PORT="${G2H_PLATFORM_HOST}:${G2H_PLATFORM_PORT}"

  echo "ENVOY PORT: $G2H_PORT"
  echo "PLATFORM_HOST: $G2H_PLATFORM_HOST"
  echo "PLATFORM_PORT: $G2H_PLATFORM_PORT"
  echo "PROXY_HOST: $G2H_PROXY_HOST"
  echo "PROXY_PORT: $G2H_PROXY_PORT"
  echo "SERVER_CERT: $G2H_SERVER_CERT"
  echo "SERVER_KEY: $G2H_SERVER_KEY"

  DOWNLOAD_URL="https://github.com/envoyproxy/envoy/releases/download/v${ENVOY_VERSION}/envoy-${ENVOY_VERSION}-linux-x86_64"
  INSTALL_DIR="/usr/local/bin"

  echo "Downloading Envoy v${ENVOY_VERSION}..."
  wget -O ${INSTALL_DIR}/envoy ${DOWNLOAD_URL}

  echo "Setting envoy bin permissions..."
  chmod +x ${INSTALL_DIR}/envoy

  echo "Verifying Envoy installation..."
  ${INSTALL_DIR}/envoy --version
  CONFIG_TEMPLATE="./config.tmpl.yaml"

  echo "Creating base envoy config template"
  create_template_file $CONFIG_TEMPLATE
  cp $CONFIG_TEMPLATE ./config.tmpl.backup.yaml
  CONFIG_FILE="/etc/envoy/envoy.yaml"

  echo "Updating Envoy configuration template with options, template at: ${CONFIG_TEMPLATE}"
  if [ -n "$G2H_PORT" ]; then
    echo "Setting port"
    $YQ_BIN -i '.static_resources.listeners[0].address.socket_address.port_value = env(G2H_PORT)' "${CONFIG_TEMPLATE}"
  fi

  # Replace the host rewrite
  $YQ_BIN -i '.static_resources.listeners[0].filter_chains[0].filters[0].typed_config.route_config.virtual_hosts[0].routes[0].route.host_rewrite_literal = strenv(G2H_PLATFORM_HOST)' "${CONFIG_TEMPLATE}"

  # Replace the tunnel target
  $YQ_BIN -i '.static_resources.listeners[1].filter_chains[0].filters[0].typed_config.tunneling_config.hostname = strenv(G2H_PLATFORM_HOST_WITH_PORT)' "${CONFIG_TEMPLATE}"

  # Replace Proxy cluster host & port
  $YQ_BIN -i '.static_resources.clusters[0].load_assignment.endpoints[0].lb_endpoints[0].endpoint.address.socket_address.address = strenv(G2H_PROXY_HOST)' "${CONFIG_TEMPLATE}"
  $YQ_BIN -i '.static_resources.clusters[0].load_assignment.endpoints[0].lb_endpoints[0].endpoint.address.socket_address.port_value = env(G2H_PROXY_PORT)' "${CONFIG_TEMPLATE}"

  # Replace loopback target sni
  $YQ_BIN -i '.static_resources.clusters[1].transport_socket.typed_config.sni = strenv(G2H_PLATFORM_HOST)' "${CONFIG_TEMPLATE}"

  # If PROXY_CREDENTIALS need to replace Proxy Authorization, otherwise remove field entirely
  if [ -n "$PROXY_CREDENTIALS" ]; then
    export PROXY_CREDENTIALS_WITH_BASIC="Basic ${PROXY_CREDENTIALS}"
    $YQ_BIN -i '.static_resources.listeners[1].filter_chains[0].filters[0].typed_config.tunneling_config.headers_to_add[0].header.value = strenv(PROXY_CREDENTIALS_WITH_BASIC)' "${CONFIG_TEMPLATE}"
  else
    $YQ_BIN eval -i 'del(.static_resources.listeners[1].filter_chains[0].filters[0].typed_config.tunneling_config.headers_to_add)' "${CONFIG_TEMPLATE}"
  fi
  # Conditionally update TLS configuration
  if [ -n "$G2H_SERVER_CERT" ] && [ -n "$G2H_SERVER_KEY" ]; then
    echo "Configuring envoy certs"
    G2H_TPA_CONFIG_SECURE="true"
    $YQ_BIN -i '.static_resources.listeners[0].filter_chains[0].transport_socket.typed_config.common_tls_context.tls_certificates[0].certificate_chain.filename = strenv(G2H_SERVER_CERT)' "${CONFIG_TEMPLATE}"
    $YQ_BIN -i '.static_resources.listeners[0].filter_chains[0].transport_socket.typed_config.common_tls_context.tls_certificates[0].private_key.filename = strenv(G2H_SERVER_KEY)' "${CONFIG_TEMPLATE}"
  else
    echo "Not configuring envoy certs"
    $YQ_BIN eval -i 'del(.static_resources.listeners[0].filter_chains[0].transport_socket)' "${CONFIG_TEMPLATE}"
  fi

  mkdir -p /etc/envoy/
  cp ${CONFIG_TEMPLATE} ${CONFIG_FILE}

  echo "Setting up Envoy as a system service..."
  cat <<EOF | tee /etc/systemd/system/envoy.service
  [Unit]
  Description=Envoy Proxy
  After=network.target

  [Service]
  Type=simple
  ExecStart=bash -c '${INSTALL_DIR}/envoy -c ${CONFIG_FILE} | tee'
  Restart=always

  [Install]
  WantedBy=multi-user.target
EOF
  
  sudo systemctl daemon-reload
  sudo systemctl enable envoy
  sudo systemctl start envoy

  echo "Envoy installation and service setup complete."

  echo "--- Next steps ---"
  echo "Update Traceable Agent config.yaml to point to this host"
  echo "global:"
  echo "  remote:"
  echo "    endpoint: THIS_MACHINE_IP_OR_HOST:PORT"
  echo "    secure: true, use false if not using cert/key"
}

create_template_file(){
  cat <<EOF > $1
static_resources:
  listeners:
    - name: main_listener
      address:
        socket_address:
          address: 0.0.0.0
          port_value: 80
      filter_chains:
        - filters:
            - name: envoy.filters.network.http_connection_manager
              typed_config:
                "@type": type.googleapis.com/envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager
                codec_type: auto
                stat_prefix: ingress_http
                access_log:
                  - name: envoy.access_loggers.file
                    typed_config:
                      "@type": type.googleapis.com/envoy.extensions.access_loggers.file.v3.FileAccessLog
                      path: "/dev/stdout"
                route_config:
                  name: local_route
                  virtual_hosts:
                    - name: app
                      domains:
                        - "*"
                      routes:
                        - match:
                            prefix: "/"
                          route:
                            host_rewrite_literal: api.traceable.ai
                            cluster: "loopback_cluster"
                http_filters:
                  - name: envoy.filters.http.grpc_http1_reverse_bridge
                    typed_config:
                      "@type": type.googleapis.com/envoy.extensions.filters.http.grpc_http1_reverse_bridge.v3.FilterConfig
                      content_type: "application/grpc"
                      withhold_grpc_frames: false
                  - name: envoy.filters.http.router
                    typed_config:
                      "@type": type.googleapis.com/envoy.extensions.filters.http.router.v3.Router
          transport_socket:
            name: envoy.transport_sockets.tls
            typed_config:
              "@type": type.googleapis.com/envoy.extensions.transport_sockets.tls.v3.DownstreamTlsContext
              common_tls_context:
                tls_certificates:
                  - certificate_chain:
                      filename: "/etc/certs/server.crt"
                    private_key:
                      filename: "/etc/certs/server.key"
    - name: api_proxy_listener
      address:
        pipe:
          path: "@/cluster_0"
      filter_chains:
        - filters:
            - name: tcp
              typed_config:
                "@type": type.googleapis.com/envoy.extensions.filters.network.tcp_proxy.v3.TcpProxy
                stat_prefix: tcp_stats
                cluster: "proxy_cluster"
                tunneling_config:
                  hostname: api.traceable.ai:443
                  headers_to_add:
                    - header:
                        key: Proxy-Authorization
                        value: ""
                      append: false
  clusters:
    - name: proxy_cluster
      connect_timeout: 0.25s
      type: strict_dns
      lb_policy: round_robin
      load_assignment:
        cluster_name: proxy_cluster
        endpoints:
          - lb_endpoints:
              - endpoint:
                  address:
                    socket_address:
                      address: 0.0.0.0
                      port_value:  20000
    - name: loopback_cluster
      connect_timeout: 5s
      upstream_connection_options:
        tcp_keepalive: {}
      type: STATIC
      load_assignment:
        cluster_name: loopback_cluster
        endpoints:
          - lb_endpoints:
              - endpoint:
                  address:
                    pipe:
                      path: "@/cluster_0"
      transport_socket:
        name: envoy.transport_sockets.tls
        typed_config:
          "@type": type.googleapis.com/envoy.extensions.transport_sockets.tls.v3.UpstreamTlsContext
          sni: api.traceable.ai
EOF
}

install_module() {
  set_distribution_id
  set_memory_configuration_key

  if [ "$PACKAGE_MANAGER" == "apt" ]
  then
    apt_setup
  else
    yum_setup
  fi

  if [ "$TPA_MAX_CPU" != "" ] || [ "$EBPF_MAX_CPU" != "" ]
  then
    check_cpu_accounting
  fi

  if [ "$INSTALL_EBPF" == "true" ]
  then
    update_ebpf_config
    checkErrorAndExitWithMessage "Error while updating eBPF configuration"
    update_ebpf_service
    checkErrorAndExitWithMessage "Error while updating eBPF service"
  elif [ "$INSTALL_SURICATA" == "true" ]
  then
    if [ "$ENABLE_TLS_CAPTURE" == "true" ]
    then
      setup_dummy_interface
      create_tls_packet_capture_service
      checkErrorAndExitWithMessage "Error while creating tls-packet-capture service"
      create_suricata_service "-i mirror0"
      checkErrorAndExitWithMessage "Error while creating suricata service"
    else
      create_suricata_service "$TLS_PACKET_CAPTURE_INTERFACE_OPTIONS"
      checkErrorAndExitWithMessage "Error while creating suricata service"
    fi
  fi

  if [ "$INSTALL_HTTP_PROXY" == "true" ]
  then
    install_http_proxy
  fi

  if [ "$INSTALL_TPA" == "true" ]
  then
    update_traceable_agent_config
    checkErrorAndExitWithMessage "Error while updating traceable agent config"
    update_traceable_agent_service
    checkErrorAndExitWithMessage "Error while updating traceable agent service"
    update_traceable_environment
    checkErrorAndExitWithMessage "Error while setting TA_ENVIRONMENT for traceable agent service"
  fi

  if [ "$PROMPT_FOR_TOKEN" == "true" ]
  then
    read_refresh_token
  fi
  update_traceable_token
  checkErrorAndExitWithMessage "Error while updating refresh token"

  systemctl daemon-reload
  checkErrorAndExitWithMessage "Error while reloading services"
  if [ "$INSTALL_TPA" == "true" ]
  then
    systemctl restart traceable
    checkErrorAndExitWithMessage "Error while restarting traceable agent service"
  fi
  if [ "$INSTALL_EBPF" == "true" ]
  then
    setup_selinux_ebpf
    systemctl restart traceable-ebpf-tracer
    checkErrorAndExitWithMessage "Error while restarting traceable-ebpf-tracer service"
  elif [ "$INSTALL_SURICATA" == "true" ]
  then
    systemctl enable suricata
    systemctl restart suricata
    checkErrorAndExitWithMessage "Error while restarting suricata service"
    if [ "$ENABLE_TLS_CAPTURE" == "true" ]
    then
      systemctl enable tls-packet-capture
      systemctl restart tls-packet-capture
      checkErrorAndExitWithMessage "Error while restarting tls-packet-capture service"
    fi
  fi
  echo "Installation complete."
}

cleanup_traceable_deployments() {
  printf "Cleaning up older Traceable deployments... "
  declare -a services=("traceable" "traceable-ebpf-tracer" "suricata" "tls-packet-capture")

  for service in "${services[@]}"
  do
    if [ "$PACKAGE_MANAGER" == "apt" ]
    then
      dpkg -r "${service}" >/dev/null 2>&1
    else
      pkgs=$(rpm -qa | grep -i "^${service}-")
      for pkg in $pkgs
      do
        echo "Removing $pkg"
        rpm -e "$pkg" >/dev/null 2>&1
      done
    fi
    systemctl disable "${service}" >/dev/null 2>&1
    systemctl stop "${service}" >/dev/null 2>&1
  done

  ip link delete mirror0 >/dev/null 2>&1
  systemctl daemon-reload >/dev/null 2>&1

  for service in "${services[@]}"
  do
    systemctl reset-failed "${service}" >/dev/null 2>&1
  done

  declare -a paths=(
    "${TPA_SERVICE_FILE}"
    "${TPA_SERVICE_DIR}"
    "${EBPF_SERVICE_DIR}"
    "${MIRROR_SERVICE_FILE}"
    "${PACKET_CAPTURE_SERVICE_FILE}"
    "/usr/bin/tls-packet-capture-wrapper"
    "/var/traceable/certs"
    "/etc/yum.repos.d/traceable.repo"
    "/etc/apt/preferences.d/traceable"
    "/etc/systemd/system/traceable.service.d"
    "/usr/share/keyrings/traceable.gpg"
    "/etc/apt/sources.list.d/traceable.list"
  )
  for path in "${paths[@]}"
  do
    rm -rf "${path}" >/dev/null 2>&1
  done
  echo "done."
}

set_package_manager
parse_input "$@"
install_dep
install_module
